@@ -147,9 +147,9 @@ def get_platform_name():
     return sys.platform
 
 
+BUILD_INFO_FILE = "./xpra/build_info.py"
 def record_build_info(is_build=True):
-    BUILD_INFO_FILE = "./xpra/build_info.py"
-
+    global BUILD_INFO_FILE
     props = get_properties(BUILD_INFO_FILE)
     if is_build:
         set_prop(props, "BUILT_BY", getpass.getuser())
@@ -194,7 +194,7 @@ def get_svn_props():
                 "LOCAL_MODIFICATIONS" : "unknown"
             }
     #find revision:
-    proc = subprocess.Popen("svnversion -n", stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    proc = subprocess.Popen("svnversion -n ..", stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     (out, _) = proc.communicate()
     if proc.returncode!=0:
         print("'svnversion -n' failed with return code %s" % proc.returncode)
@@ -264,6 +264,8 @@ def has_src_info():
     return os.path.exists(SRC_INFO_FILE) and os.path.isfile(SRC_INFO_FILE)
 
 def main():
+    if not has_src_info():
+        record_src_info()
     record_build_info(True)
 
 
