@@ -9,20 +9,20 @@ import os
 import sys
 import socket
 import binascii
+import string
 from xpra.gtk_common.gobject_compat import import_gobject, import_glib
 gobject = import_gobject()
 
 from xpra.log import Logger
 log = Logger("client")
 
-from xpra.net.protocol import Protocol, use_lz4, use_rencode, get_network_caps
+from xpra.net.protocol import Protocol, get_network_caps
 from xpra.scripts.config import ENCRYPTION_CIPHERS
-from xpra.scripts.main import SOCKET_TIMEOUT
 from xpra.version_util import version_compat_check, get_version_info, get_platform_info, local_version
 from xpra.platform.features import GOT_PASSWORD_PROMPT_SUGGESTION
 from xpra.platform.info import get_name
 from xpra.os_util import get_hex_uuid, get_machine_id, get_user_uuid, load_binary_file, SIGNAMES, strtobytes, bytestostr
-from xpra.util import typedict, xor
+from xpra.util import b, ss, typedict, updict, xor, repr_ellipsized, nonl, disconnect_is_an_error
 
 EXIT_OK = 0
 EXIT_CONNECTION_LOST = 1
@@ -40,19 +40,11 @@ EXIT_UNSUPPORTED = 12
 EXIT_REMOTE_ERROR = 13
 
 
-DEFAULT_TIMEOUT = (10 + SOCKET_TIMEOUT)*1000        #in millis
+EXTRA_TIMEOUT = 10
 ALLOW_UNENCRYPTED_PASSWORDS = os.environ.get("XPRA_ALLOW_UNENCRYPTED_PASSWORDS", "0")=="1"
 DETECT_LEAKS = os.environ.get("XPRA_DETECT_LEAKS", "0")=="1"
 
 
-if sys.version < '3':
-    def b(x):
-        return x
-else:
-    def b(x):
-        return bytes(x, 'UTF-8')
-
-
 class XpraClientBase(object):
     """ Base class for Xpra clients.
         Provides the glue code for:
@@ -149,10 +141,24 @@ class XpraClientBase(object):
             sys.stderr.flush()
             signal.signal(signal.SIGINT, deadly_signal)
             signal.signal(signal.SIGTERM, deadly_signal)
-            self.timeout_add(0, self.quit, 128 + signum)
+            self.timeout_add(0, self.disconnect_and_quit, 128 + signum, "exit on signal %s" % SIGNAMES.get(signum, signum))
         signal.signal(signal.SIGINT, app_signal)
         signal.signal(signal.SIGTERM, app_signal)
 
+    def disconnect_and_quit(self, exit_code, reason):
+        #try to tell the server we're going, then quit
+        log("disconnect_and_quit(%s, %s)", exit_code, reason)
+        p = self._protocol
+        if p and p._closed:
+            self.quit(exit_code)
+            return
+        def do_quit():
+            log("disconnect_and_quit: do_quit()")
+            self.quit(exit_code)
+        if p:
+            p.flush_then_close(["disconnect", reason], done_callback=do_quit)
+        self.timeout_add(1000, do_quit)
+
 
     def client_type(self):
         #overriden in subclasses!
@@ -168,20 +174,30 @@ class XpraClientBase(object):
         self._protocol.large_packets.append("server-settings")
         self._protocol.set_compression_level(self.compression_level)
         self._protocol.receive_aliases.update(self._aliases)
+        self._protocol.enable_default_encoder()
+        self._protocol.enable_default_compressor()
         self.have_more = self._protocol.source_has_more
+        if conn.timeout>0:
+            self.timeout_add((conn.timeout + EXTRA_TIMEOUT) * 1000, self.verify_connected)
 
     def init_packet_handlers(self):
         self._packet_handlers = {
             "hello": self._process_hello,
             }
         self._ui_packet_handlers = {
-            "challenge": self._process_challenge,
-            "disconnect": self._process_disconnect,
-            "set_deflate": self._process_set_deflate,
-            Protocol.CONNECTION_LOST: self._process_connection_lost,
-            Protocol.GIBBERISH: self._process_gibberish,
+            "challenge":                self._process_challenge,
+            "disconnect":               self._process_disconnect,
+            "set_deflate":              self._process_set_deflate,
+            Protocol.CONNECTION_LOST:   self._process_connection_lost,
+            Protocol.GIBBERISH:         self._process_gibberish,
+            Protocol.INVALID:           self._process_invalid,
             }
 
+    def init_authenticated_packet_handlers(self):
+        #for subclasses to override
+        pass
+
+
     def init_aliases(self):
         packet_types = list(self._packet_handlers.keys())
         packet_types += list(self._ui_packet_handlers.keys())
@@ -206,7 +222,6 @@ class XpraClientBase(object):
                 hello["challenge_client_salt"] = client_salt
         log.debug("send_hello(%s) packet=%s", binascii.hexlify(b(challenge_response or "")), hello)
         self.send("hello", hello)
-        self.timeout_add(DEFAULT_TIMEOUT, self.verify_connected)
 
     def verify_connected(self):
         if self.server_capabilities is None:
@@ -230,8 +245,10 @@ class XpraClientBase(object):
                 })
         if self.display:
             capabilities["display"] = self.display
-        capabilities.update(get_platform_info("platform"))
-        capabilities.update(get_version_info("build"))
+        def up(prefix, d):
+            updict(capabilities, prefix, d)
+        up("platform",  get_platform_info())
+        up("build",     get_version_info())
         mid = get_machine_id()
         if mid:
             capabilities["machine_id"] = mid
@@ -331,18 +348,32 @@ class XpraClientBase(object):
         self.quit(exit_code)
 
     def _process_disconnect(self, packet):
-        if len(packet)==2:
-            info = packet[1]
-        else:
-            info = packet[1:]
-        e = EXIT_OK
+        #ie: ("disconnect", "version error", "incompatible version")
+        reason = ss(packet[1])
+        info = packet[2:]
+        s = nonl(reason)
+        if len(info):
+            s += " (%s)" % (", ".join([nonl(ss(x)) for x in info]))
         if self.server_capabilities is None or len(self.server_capabilities)==0:
             #server never sent hello to us - so disconnect is an error
             #(but we don't know which one - the info message may help)
+            log.warn("server failure: disconnected before the session could be established")
             e = EXIT_FAILURE
-        self.warn_and_quit(e, "server requested disconnect: %s" % info)
+        elif disconnect_is_an_error(reason):
+            log.warn("server failure: %s", reason)
+            e = EXIT_FAILURE
+        else:
+            e = EXIT_OK
+        self.warn_and_quit(e, "server requested disconnect: %s" % s)
 
     def _process_connection_lost(self, packet):
+        p = self._protocol
+        if p and p.input_raw_packetcount==0:
+            props = p.get_info()
+            c = props.get("compression", "unknown")
+            e = props.get("encoder", "unknown")
+            log.warn("disconnected without receiving a single packet, not an xpra server?")
+            log.warn("(maybe it does not support '%s' compression or '%s' packet encoding)", c, e)
         self.warn_and_quit(EXIT_CONNECTION_LOST, "Connection lost")
 
     def _process_challenge(self, packet):
@@ -433,7 +464,7 @@ class XpraClientBase(object):
         try:
             self.server_capabilities = typedict(packet[1])
             log("processing hello from server: %s", self.server_capabilities)
-            self.parse_server_capabilities(self.server_capabilities)
+            self.server_connection_established()
         except Exception, e:
             log.info("error in hello packet", exc_info=True)
             self.warn_and_quit(EXIT_FAILURE, "error processing hello packet from server: %s" % e)
@@ -444,7 +475,28 @@ class XpraClientBase(object):
             v = bytestostr(v)
         return v
 
-    def parse_server_capabilities(self, c):
+
+    def server_connection_established(self):
+        log("server_connection_established()")
+        if not self.parse_version_capabilities():
+            log("server_connection_established() failed version capabilities")
+            return False
+        if not self.parse_server_capabilities():
+            log("server_connection_established() failed server capabilities")
+            return False
+        if not self.parse_network_capabilities():
+            log("server_connection_established() failed network capabilities")
+            return False
+        if not self.parse_encryption_capabilities():
+            log("server_connection_established() failed encryption capabilities")
+            return False
+        log("server_connection_established() adding authenticated packet handlers")
+        self.init_authenticated_packet_handlers()
+        return True
+
+
+    def parse_version_capabilities(self):
+        c = self.server_capabilities
         self._remote_machine_id = c.strget("machine_id")
         self._remote_uuid = c.strget("uuid")
         self._remote_version = c.strget("version")
@@ -466,11 +518,20 @@ class XpraClientBase(object):
         if verr is not None:
             self.warn_and_quit(EXIT_INCOMPATIBLE_VERSION, "incompatible remote version '%s': %s" % (self._remote_version, verr))
             return False
+        return True
 
-        if use_rencode and c.boolget("rencode"):
-            self._protocol.enable_rencode()
-        if use_lz4 and c.boolget("lz4") and self.compression_level==1:
-            self._protocol.enable_lz4()
+    def parse_server_capabilities(self):
+        return True
+
+    def parse_network_capabilities(self):
+        c = self.server_capabilities
+        if not self._protocol.enable_encoder_from_caps(c):
+            return False
+        self._protocol.enable_compressor_from_caps(c)
+        return True
+
+    def parse_encryption_capabilities(self):
+        c = self.server_capabilities
         if self.encryption:
             #server uses a new cipher after second hello:
             key = self.get_encryption_key()
@@ -478,10 +539,6 @@ class XpraClientBase(object):
             if not self.set_server_encryption(c, key):
                 return False
         self._protocol.send_aliases = c.dictget("aliases", {})
-        if self.pings:
-            self.timeout_add(1000, self.send_ping)
-        else:
-            self.timeout_add(10*1000, self.send_ping)
         return True
 
     def _process_set_deflate(self, packet):
@@ -489,19 +546,32 @@ class XpraClientBase(object):
         pass
 
     def _process_gibberish(self, packet):
-        (_, data) = packet
-        log.info("Received uninterpretable nonsense: %s", repr(data))
+        (_, message, data) = packet
+        p = self._protocol
+        if (p and p.input_packetcount>0) or len([c for c in data if c not in string.printable])>0:
+            log.info("Received uninterpretable nonsense: %s", message)
+            log.info(" packet no %i data: %s", p.input_packetcount, repr_ellipsized(data))
+        else:
+            #looks like the first packet back is just text, print it:
+            log.info("Failed to connect: %s", repr_ellipsized(data.strip("\n").strip("\r")))
         if str(data).find("assword")>0:
-            log.warn("Your ssh program appears to be asking for a password."
+            self.warn_and_quit(EXIT_SSH_FAILURE,
+                              "Your ssh program appears to be asking for a password."
                              + GOT_PASSWORD_PROMPT_SUGGESTION)
-            self.quit(EXIT_SSH_FAILURE)
         elif str(data).find("login")>=0:
-            log.warn("Your ssh program appears to be asking for a username.\n"
+            self.warn_and_quit(EXIT_SSH_FAILURE,
+                             "Your ssh program appears to be asking for a username.\n"
                              "Perhaps try using something like 'ssh:USER@host:display'?")
-            self.quit(EXIT_SSH_FAILURE)
         else:
             self.quit(EXIT_PACKET_FAILURE)
 
+    def _process_invalid(self, packet):
+        (_, message, data) = packet
+        log.info("Received invalid packet: %s", message)
+        log(" data: %s", repr_ellipsized(data))
+        self.quit(EXIT_PACKET_FAILURE)
+
+
     def process_packet(self, proto, packet):
         try:
             handler = None
