@@ -22,10 +22,10 @@ class ClientTray(ClientWidgetBase):
 
     def __init__(self, client, wid, w, h, tray_widget, mmap_enabled, mmap_area):
         log("ClientTray%s", (client, wid, w, h, tray_widget, mmap_enabled, mmap_area))
-        ClientWidgetBase.__init__(self, client, wid)
+        ClientWidgetBase.__init__(self, client, wid, True)
         self.tray_widget = tray_widget
-        self._has_alpha = True
         self._geometry = None
+        self._window_alpha = True
         self.group_leader = None
 
         self.mmap_enabled = mmap_enabled
@@ -34,6 +34,9 @@ class ClientTray(ClientWidgetBase):
         self.new_backing(w, h)
         self.idle_add(self.reconfigure)
 
+    def get_backing_class(self):
+        return TrayBacking
+
     def is_OR(self):
         return True
 
@@ -145,16 +148,30 @@ class TrayBacking(GTKWindowBacking):
         we can use them with the real widget.
     """
 
+    #keep it simple: only accept 32-bit RGB(X),
+    #all tray implementations support alpha
+    RGB_MODES = ["RGBA", "RGBX"]
+    HAS_ALPHA = True
+
     def __init__(self, wid, w, h, has_alpha, data=None):
         self.data = data
-        GTKWindowBacking.__init__(self, wid)
+        GTKWindowBacking.__init__(self, wid, True)
+        self._backing = object()    #pretend we have a backing structure
 
-    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        log("TrayBacking._do_paint_rgb24%s", ("%s bytes" % len(img_data), x, y, width, height, rowstride, options, callbacks))
+    def get_encoding_properties(self):
+        #override so we skip all csc caps:
+        return {
+                "encodings.rgb_formats" : self.RGB_MODES,
+                "encoding.transparency" : True
+               }
+
+
+    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
+        log("TrayBacking._do_paint_rgb24%s", ("%s bytes" % len(img_data), x, y, width, height, rowstride, options))
         self.data = ["rgb24", width, height, rowstride, img_data[:]]
         return True
 
-    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        log("TrayBacking._do_paint_rgb32%s", ("%s bytes" % len(img_data), x, y, width, height, rowstride, options, callbacks))
+    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options):
+        log("TrayBacking._do_paint_rgb32%s", ("%s bytes" % len(img_data), x, y, width, height, rowstride, options))
         self.data = ["rgb32", width, height, rowstride, img_data[:]]
         return True
