@@ -100,12 +100,6 @@ if OPENGL_DEBUG:
 from ctypes import c_char_p
 
 
-#support for memory views requires Python 2.7 and PyOpenGL 3.1
-memoryview_type = None
-if sys.version_info[:2]>=(2,7) and OpenGL_version.__version__.split('.')[:2]>=['3','1']:
-    memoryview_type = memoryview
-
-
 # Texture number assignment
 #  1 = Y plane
 #  2 = U plane
@@ -465,13 +459,6 @@ class GLPixmapBacking(GTK2WindowBacking):
             log("%s._do_paint_rgb(..) drawable is not set!", self)
             return False
 
-        #deal with buffers uploads by wrapping them if we can, or copy to a string:
-        if type(img_data)==buffer:
-            if memoryview_type is not None:
-                img_data = memoryview_type(img_data)
-            else:
-                img_data = str(img_data)
-
         try:
             self.set_rgb_paint_state()
 
@@ -534,28 +521,8 @@ class GLPixmapBacking(GTK2WindowBacking):
         return True
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
-        clone = True
-        #we can only use zero copy upload if the image is "thread_safe"
-        #(dec_avcodec2 is not safe...)
-        #and if the data is a memoryview (or if we can wrap it in one)
-        if memoryview_type is not None and img.is_thread_safe():
-            pixels = img.get_pixels()
-            assert len(pixels)==3, "invalid number of planes: %s" % len(pixels)
-            plane_types = list(set([type(plane) for plane in img.get_pixels()]))
-            if len(plane_types)==1:
-                plane_type = plane_types[0]
-                if plane_type==memoryview_type:
-                    clone = False
-                elif plane_type in (str, buffer):
-                    #wrap with a memoryview that pyopengl can use:
-                    views = []
-                    for i in range(3):
-                        views.append(memoryview_type(pixels[i]))
-                    img.set_pixels(views)
-                    clone = False
-        if clone:
-            #copy so the data will be usable (usually a str)
-            img.clone_pixel_data()
+        #copy so the data will be usable (usually a str)
+        img.clone_pixel_data()
         gobject.idle_add(self.gl_paint_planar, img, x, y, enc_width, enc_height, width, height, callbacks)
 
     def gl_paint_planar(self, img, x, y, enc_width, enc_height, width, height, callbacks):
