@@ -190,7 +190,7 @@ class XpraClient(GTKXpraClient):
     def process_clipboard_packet(self, packet):
         clipboardlog("process_clipboard_packet(%s) level=%s", packet, gtk.main_level())
         #check for clipboard loops:
-        if gtk.main_level()>=10:
+        if gtk.main_level()>=40:
             clipboardlog.warn("loop nesting too deep: %s", gtk.main_level())
             clipboardlog.warn("you may have a clipboard forwarding loop, disabling the clipboard")
             self.clipboard_enabled = False
@@ -386,7 +386,7 @@ class XpraClient(GTKXpraClient):
                 #use default:
                 cursor = gdk.Cursor(gtk.gdk.X_CURSOR)
         for gtkwindow in gtkwindows:
-            if gtk.gtk_version>=(2,14):
+            if hasattr(gtkwindow, "get_window"):
                 gdkwin = gtkwindow.get_window()
             else:
                 gdkwin = gtkwindow.window
