@@ -11,7 +11,7 @@ import cairo
 from xpra.log import Logger
 log = Logger("paint")
 
-from xpra.client.gtk2.window_backing import GTK2WindowBacking, HAS_ALPHA
+from xpra.client.gtk2.window_backing import GTK2WindowBacking
 
 
 """
@@ -19,13 +19,19 @@ Backing using a gdk.Pixmap
 """
 class PixmapBacking(GTK2WindowBacking):
 
+    RGB_MODES = ["RGB", "RGBX", "RGBA"]
+
     def __repr__(self):
         return "PixmapBacking(%s)" % self._backing
 
+    def __init__(self, wid, w, h, has_alpha):
+        self._backing = None
+        GTK2WindowBacking.__init__(self, wid, has_alpha)
+
     def init(self, w, h):
         old_backing = self._backing
         assert w<32768 and h<32768, "dimensions too big: %sx%s" % (w, h)
-        if self._has_alpha and HAS_ALPHA:
+        if self._alpha_enabled:
             self._backing = gdk.Pixmap(None, w, h, 32)
             screen = self._backing.get_screen()
             rgba = screen.get_rgba_colormap()
@@ -33,7 +39,8 @@ class PixmapBacking(GTK2WindowBacking):
                 self._backing.set_colormap(rgba)
             else:
                 #cannot use transparency
-                self._has_alpha = False
+                log.warn("cannot use transparency: no RGBA colormap!")
+                self._alpha_enabled = False
                 self._backing = gdk.Pixmap(gdk.get_default_root_window(), w, h)
         else:
             self._backing = gdk.Pixmap(gdk.get_default_root_window(), w, h)
@@ -64,18 +71,15 @@ class PixmapBacking(GTK2WindowBacking):
             cr.rectangle(0, 0, w, h)
             cr.fill()
 
-    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        if self._backing is None:
-            return  False
+    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
+        img_data = self.img_data_tobytes(img_data)
         gc = self._backing.new_gc()
         self._backing.draw_rgb_image(gc, x, y, width, height, gdk.RGB_DITHER_NONE, img_data, rowstride)
         return True
 
-    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options, callbacks):
+    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options):
         #log.info("do_paint_rgb32(%s bytes, %s, %s, %s, %s, %s, %s, %s) backing depth=%s", len(img_data), x, y, width, height, rowstride, options, callbacks, self._backing.get_depth())
         #log.info("data head=%s", [hex(ord(v))[2:] for v in list(img_data[:500])])
-        if self._backing is None:
-            return  False
         rgba = self.unpremultiply(img_data)
         pixbuf = gdk.pixbuf_new_from_data(rgba, gtk.gdk.COLORSPACE_RGB, True, 8, width, height, rowstride)
         cr = self._backing.cairo_create()
