@@ -9,6 +9,9 @@ import os.path
 import sys
 import signal
 
+from xpra.platform.gui import init as gui_init
+gui_init()
+
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_gobject, import_pango
 
 gtk = import_gtk()
@@ -23,6 +26,7 @@ from xpra.gtk_common.gtk_util import set_tooltip_text, add_close_accel, scaled_i
 from xpra.client.gtk_base.about import about
 from xpra.client.client_base import SIGNAMES
 from xpra.platform.paths import get_icon_dir
+from xpra.platform.info import get_user_info
 from xpra.util import nonl, updict
 from xpra.log import Logger, enable_debug_for
 log = Logger("util")
@@ -118,6 +122,8 @@ class BugReport(object):
                     "host"          : get_host_info(),
                     "paths"         : get_path_info(),
                     "gtk"           : get_gtk_version_info(),
+                    "user"          : get_user_info(),
+                    "env"           : os.environ,
                     }.items():
                 updict(d, k, v)
             return d
@@ -288,7 +294,8 @@ class BugReport(object):
                 info = zipfile.ZipInfo(cfile, date_time=time.localtime(time.time()))
                 info.compress_type = zipfile.ZIP_DEFLATED
                 #very poorly documented:
-                info.external_attr = 0o644 << 16
+                #info.external_attr = 0o644 << 16
+                info.external_attr = 420 << 16
                 info.comment = tooltip
                 zf.writestr(info, s, compress_type=zipfile.ZIP_DEFLATED)
         finally:
@@ -300,9 +307,8 @@ def main():
     gtk_main_quit_on_fatal_exceptions_enable()
 
     from xpra.platform import init as platform_init
-    from xpra.platform.gui import init as gui_init, ready as gui_ready
+    from xpra.platform.gui import ready as gui_ready
     platform_init("Xpra-Bug-Report", "Xpra Bug Report")
-    gui_init()
 
     #logging init:
     if "-v" in sys.argv:
