@@ -15,6 +15,9 @@ import sys
 import shlex
 import signal
 
+from xpra.platform.gui import init as gui_init
+gui_init()
+
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_gobject, import_pango
 
 gtk = import_gtk()
@@ -692,7 +695,7 @@ def main():
     gtk_main_quit_on_fatal_exceptions_enable()
 
     from xpra.platform import init as platform_init
-    from xpra.platform.gui import init as gui_init, ready as gui_ready
+    from xpra.platform.gui import ready as gui_ready
     platform_init("Xpra-Launcher", "Xpra Connection Launcher")
     gui_init()
 
