@@ -30,7 +30,7 @@ from xpra.client.client_tray import ClientTray
 from xpra.client.keyboard_helper import KeyboardHelper
 from xpra.platform import set_application_name
 from xpra.platform.features import MMAP_SUPPORTED, SYSTEM_TRAY_SUPPORTED, CLIPBOARD_WANT_TARGETS, CLIPBOARD_GREEDY, CLIPBOARDS
-from xpra.platform.gui import init as gui_init, ready as gui_ready, get_native_notifier_classes, get_native_tray_classes, get_native_system_tray_classes, get_native_tray_menu_helper_classes, ClientExtras
+from xpra.platform.gui import ready as gui_ready, get_native_notifier_classes, get_native_tray_classes, get_native_system_tray_classes, get_native_tray_menu_helper_classes, ClientExtras
 from xpra.codecs.codec_constants import get_PIL_decodings
 from xpra.codecs.loader import codec_versions, has_codec, get_codec, PREFERED_ENCODING_ORDER, ALL_NEW_ENCODING_NAMES_TO_OLD, OLD_ENCODING_NAMES_TO_NEW, PROBLEMATIC_ENCODINGS
 from xpra.codecs.video_helper import getVideoHelper, NO_GFX_CSC_OPTIONS
@@ -69,7 +69,6 @@ class UIXpraClient(XpraClientBase):
     def __init__(self):
         XpraClientBase.__init__(self)
         log.info("xpra client version %s" % XPRA_VERSION)
-        gui_init()
         self.start_time = time.time()
         self._window_to_id = {}
         self._id_to_window = {}
@@ -270,14 +269,16 @@ class UIXpraClient(XpraClientBase):
             self.tray = self.setup_xpra_tray(opts.tray_icon)
             if self.tray:
                 tray_icon_filename = self.tray.get_tray_icon_filename(tray_icon_filename)
+                #keep tray widget hidden until:
+                self.tray.hide()
                 if opts.delay_tray:
-                    self.tray.hide()
                     def show_tray(*args):
                         traylog("first ui received, showing tray %s", self.tray)
                         self.tray.show()
                     self.connect("first-ui-received", show_tray)
                 else:
-                    self.tray.show()
+                    #show when the main loop is running:
+                    self.idle_add(self.tray.show)
 
         if self.client_supports_notifications:
             self.notifier = self.make_notifier()
@@ -551,7 +552,8 @@ class UIXpraClient(XpraClientBase):
             traylog("xpra_tray_exit(%s)", args)
             self.disconnect_and_quit(0, CLIENT_EXIT)
         def xpra_tray_geometry(*args):
-            traylog("xpra_tray_geometry%s geometry=%s", args, tray.get_geometry())
+            if tray:
+                traylog("xpra_tray_geometry%s geometry=%s", args, tray.get_geometry())
         menu = None
         if self.menu_helper:
             menu = self.menu_helper.build()
@@ -590,14 +592,24 @@ class UIXpraClient(XpraClientBase):
                 modifiers = self.get_current_modifiers()
                 buttons = []
                 self.send_mouse_position(["pointer-position", wid, pointer, modifiers, buttons])
-        def tray_geometry(*args):
+        def do_tray_geometry(*args):
             #tell the "ClientTray" where it now lives
             #which should also update the location on the server if it has changed
             tray = self._id_to_window.get(wid)
-            geom = tray_widget.get_geometry()
-            traylog("tray_geometry(%s) geometry=%s tray=%s", args, geom, tray)
+            if tray_widget:
+                geom = tray_widget.get_geometry()
+            else:
+                geom = None
+            traylog("tray_geometry(%s) widget=%s, geometry=%s tray=%s", args, tray_widget, geom, tray)
             if tray and geom:
                 tray.move_resize(*geom)
+        def tray_geometry(*args):
+            #the tray widget may still be None if we haven't returned from make_system_tray yet,
+            #in which case we will check the geometry a little bit later:
+            if tray_widget:
+                do_tray_geometry(*args)
+            else:
+                self.idle_add(do_tray_geometry, *args)
         def tray_exit(*args):
             traylog("tray_exit(%s)", args)
         tray_widget = self.make_system_tray(None, title, None, tray_geometry, tray_click, tray_mouseover, tray_exit)
@@ -705,16 +717,6 @@ class UIXpraClient(XpraClientBase):
         return self.keyboard_helper.mask_to_names(mask)
 
 
-    def set_default_window_icon(self, window_icon):
-        if not window_icon:
-            window_icon = self.get_icon_filename("xpra.png")
-        if window_icon and os.path.exists(window_icon):
-            try:
-                self.do_set_window_icon(window_icon)
-            except Exception, e:
-                windowlog.error("failed to set window icon %s: %s", window_icon, e)
-
-
     def send_focus(self, wid):
         focuslog("send_focus(%s)", wid)
         self.send("focus", wid, self.get_current_modifiers())
