@@ -9,7 +9,39 @@ from xpra.log import Logger
 log = Logger("util")
 
 
-LOSSY_PIXEL_FORMATS = ("YUV420P", "YUV422P")
+def do_get_PIL_codings(PIL, attr="SAVE"):
+    if PIL is None:
+        return []
+    pi = PIL.Image
+    pi.init()
+    avalue = getattr(pi, attr)
+    log("PIL.Image.%s=%s", attr, avalue)
+    encodings = []
+    test_options = ["png", "png/L", "png/P", "jpeg", "webp"]
+    if attr=="OPEN":
+        #don't open 'webp' images as this leaks memory in
+        #all versions of PIL and Pillow so far!
+        test_options.remove("webp")
+    for encoding in test_options:
+        #strip suffix (so "png/L" -> "png")
+        stripped = encoding.split("/")[0].upper()
+        if stripped in avalue:
+            log("PIL.Image can %s %s", attr, encoding)
+            encodings.append(encoding)
+    log("do_get_PIL_codings(%s, %s)=%s", PIL, attr, encodings)
+    return encodings
+
+def get_PIL_encodings(PIL):
+    return do_get_PIL_codings(PIL, "SAVE")
+
+def get_PIL_decodings(PIL):
+    return do_get_PIL_codings(PIL, "OPEN")
+
+
+#value: how much smaller the output is
+LOSSY_PIXEL_FORMATS = {"YUV420P" : 2,
+                       "YUV422P" : 1.5
+                       }
 
 PIXEL_SUBSAMPLING = {
          "YUV420P"   : ((1, 1), (2, 2), (2, 2)),
@@ -117,7 +149,7 @@ class codec_spec(object):
 
 
     def get_instance_count(self):
-        return len(self.instances.keys())
+        return len(list(self.instances.keys()))
 
     def to_dict(self):
         d = {}
@@ -175,3 +207,26 @@ class video_codec_spec(codec_spec):
         self.encoding = encoding                        #ie: "h264"
         self.output_colorspaces = output_colorspaces    #ie: ["YUV420P" : "YUV420P", ...]
         self._exported_fields += ["encoding", "output_colorspaces"]
+
+
+def main():
+    from xpra.platform import init, clean
+    try:
+        init("Codec-Constants", "Codec Constants Info")
+        import sys
+        if "-v" in sys.argv or "--verbose" in sys.argv:
+            log.enable_debug()
+
+        try:
+            from PIL import Image   #@UnresolvedImport @UnusedImport
+            import PIL              #@UnresolvedImport
+        except:
+            PIL = None
+        print("PIL encodings: %s" % ", ".join(get_PIL_encodings(PIL)))
+        print("PIL decodings: %s" % ", ".join(get_PIL_decodings(PIL)))
+    finally:
+        #this will wait for input on win32:
+        clean()
+
+if __name__ == "__main__":
+    main()
