@@ -8,7 +8,7 @@ from xpra.platform.darwin.osx_menu import getOSXMenuHelper
 from xpra.platform.paths import get_icon
 
 from xpra.log import Logger
-log = Logger("osx")
+log = Logger("osx", "events")
 
 
 exit_cb = None
@@ -72,6 +72,56 @@ def system_bell(*args):
     Snd.SysBeep(1)
     return True
 
+try:
+    import Quartz.CoreGraphics as CG    #@UnresolvedImport
+    ALPHA = {
+             CG.kCGImageAlphaNone                  : "AlphaNone",
+             CG.kCGImageAlphaPremultipliedLast     : "PremultipliedLast",
+             CG.kCGImageAlphaPremultipliedFirst    : "PremultipliedFirst",
+             CG.kCGImageAlphaLast                  : "Last",
+             CG.kCGImageAlphaFirst                 : "First",
+             CG.kCGImageAlphaNoneSkipLast          : "SkipLast",
+             CG.kCGImageAlphaNoneSkipFirst         : "SkipFirst",
+       }
+except:
+    CG = None
+
+
+def get_CG_imagewrapper():
+    from xpra.codecs.image_wrapper import ImageWrapper
+    assert CG, "cannot capture without Quartz.CoreGraphics"
+    #region = CG.CGRectMake(0, 0, 100, 100)
+    region = CG.CGRectInfinite
+    image = CG.CGWindowListCreateImage(region,
+                CG.kCGWindowListOptionOnScreenOnly,
+                CG.kCGNullWindowID,
+                CG.kCGWindowImageDefault)
+    width = CG.CGImageGetWidth(image)
+    height = CG.CGImageGetHeight(image)
+    bpc = CG.CGImageGetBitsPerComponent(image)
+    bpp = CG.CGImageGetBitsPerPixel(image)
+    rowstride = CG.CGImageGetBytesPerRow(image)
+    alpha = CG.CGImageGetAlphaInfo(image)
+    alpha_str = ALPHA.get(alpha, alpha)
+    log("get_CG_imagewrapper(..) image size: %sx%s, bpc=%s, bpp=%s, rowstride=%s, alpha=%s", width, height, bpc, bpp, rowstride, alpha_str)
+    prov = CG.CGImageGetDataProvider(image)
+    argb = CG.CGDataProviderCopyData(prov)
+    return ImageWrapper(0, 0, width, height, argb, "BGRX", 24, rowstride)
+
+def take_screenshot():
+    log("grabbing screenshot")
+    from PIL import Image
+    from xpra.os_util import StringIOClass
+    image = get_CG_imagewrapper()
+    w = image.get_width()
+    h = image.get_height()
+    img = Image.frombuffer("RGB", (w, h), image.get_pixels(), "raw", image.get_pixel_format(), image.get_rowstride())
+    buf = StringIOClass()
+    img.save(buf, "PNG")
+    data = buf.getvalue()
+    buf.close()
+    return w, h, "png", image.get_rowstride(), data
+
 
 try:
     import Foundation                           #@UnresolvedImport
@@ -97,10 +147,16 @@ class NotificationHandler(NSObject):
 
 
 class ClientExtras(object):
-    def __init__(self, client, blocking=False):
+    def __init__(self, client, opts, blocking=False):
+        log("ClientExtras.__init__(%s, %s, %s) swap_keys=%s", client, opts, blocking, opts.swap_keys)
         self.client = client
         self.blocking = blocking
         self.setup_event_loop()
+        if opts and client:
+            log("setting swap_keys=%s using %s", opts.swap_keys, client.keyboard_helper)
+            if client.keyboard_helper and client.keyboard_helper.keyboard:
+                log("%s.swap_keys=%s", client.keyboard_helper.keyboard, opts.swap_keys)
+                client.keyboard_helper.keyboard.swap_keys = opts.swap_keys
 
     def cleanup(self):
         self.client = None
@@ -116,6 +172,9 @@ class ClientExtras(object):
 
     def setup_event_loop(self):
         if NSWorkspace is None:
+            self.notificationCenter = None
+            self.handler = None
+            log.warn("event loop not started")
             return
         ws = NSWorkspace.sharedWorkspace()
         self.notificationCenter = ws.notificationCenter()
@@ -139,7 +198,7 @@ class ClientExtras(object):
 def main():
     from xpra.platform import init
     init("OSX Extras")
-    ClientExtras(None, True)
+    ClientExtras(None, None, True)
 
 
 if __name__ == "__main__":
