@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -16,3 +16,7 @@ GOT_PASSWORD_PROMPT_SUGGESTION = \
 CLIPBOARDS=["CLIPBOARD"]
 CLIPBOARD_GREEDY = True
 CLIPBOARD_NATIVE_CLASS = ("xpra.clipboard.translated_clipboard", "TranslatedClipboardProtocolHelper", {})
+
+#these don't make sense on win32:
+DEFAULT_PULSEAUDIO_COMMAND = ""
+DEFAULT_XVFB_COMMAND = ""
