@@ -7,6 +7,7 @@
 
 from xpra.platform.keyboard_base import KeyboardBase
 from xpra.keyboard.layouts import WIN32_LAYOUTS
+from xpra.gtk_common.keymap import KEY_TRANSLATIONS
 from xpra.log import Logger
 log = Logger("keyboard", "win32")
 
@@ -23,6 +24,12 @@ class Keyboard(KeyboardBase):
         self.emulate_altgr = False
         self.num_lock_modifier = None
         self.last_key_event_sent = None
+        #workaround for "period" vs "KP_Decimal" with gtk2 (see ticket #586):
+        #translate "period" with keyval=46 and keycode=110 to KP_Decimal:
+        KEY_TRANSLATIONS[("period",     46,     110)]   = "KP_Decimal"
+        #workaround for "fr" keyboards, which use a different key name under X11:
+        KEY_TRANSLATIONS[("dead_tilde", 65107,  50)]    = "asciitilde"
+        KEY_TRANSLATIONS[("dead_grave", 65104,  55)]    = "grave"
 
     def set_modifier_mappings(self, mappings):
         KeyboardBase.set_modifier_mappings(self, mappings)
@@ -74,6 +81,7 @@ class Keyboard(KeyboardBase):
 
     def get_layout_spec(self):
         layout = None
+        layouts = []
         variant = None
         variants = None
         try:
@@ -84,9 +92,11 @@ class Keyboard(KeyboardBase):
                 log("found keyboard layout '%s' with variants=%s, code '%s' for kbid=%s", layout, variants, code, kbid)
             if not layout:
                 log("unknown keyboard layout for kbid: %s", kbid)
+            else:
+                layouts.append(layout)
         except Exception, e:
             log.error("failed to detect keyboard layout: %s", e)
-        return layout,variant,variants
+        return layout,layouts,variant,variants
 
     def get_keyboard_repeat(self):
         try:
@@ -113,7 +123,6 @@ class Keyboard(KeyboardBase):
             So we just ignore those and rely on the list of "modifiers" passed
             with each keypress to let the server set them for us when needed.
         """
-        log("win32.process_key_event(%s, %s, %s)", send_key_action_cb, wid, key_event)
         if key_event.keyval==2**24-1 and key_event.keyname=="VoidSymbol":
             return
         #self.modifier_mappings = None       #{'control': [(37, 'Control_L'), (105, 'Control_R')], 'mod1':
@@ -135,14 +144,6 @@ class Keyboard(KeyboardBase):
                         last_key_event.pressed = False
                         KeyboardBase.process_key_event(self, send_key_action_cb, last_wid, last_key_event)
                 self.AltGr_modifiers(key_event.modifiers, not key_event.pressed)
-        if key_event.keyname=="period" and key_event.keyval==46 and key_event.keycode==110:
-            log("patched 'period' keyname to 'KP_Decimal'")
-            key_event.keyname = "KP_Decimal"
-        elif key_event.keyname=="dead_tilde" and key_event.keyval==65107 and key_event.keycode==50:
-            log("patched 'dead_tilde' keyname to 'asciitilde'")
-            key_event.keyname = "asciitilde"
-        elif key_event.keyname=="dead_grave" and key_event.keyval==65104 and key_event.keycode==55:
-            log("patched 'dead_grave' keyname to 'grave'")
-            key_event.keyname = "grave"
         self.last_key_event_sent = (wid, key_event)
-        send_key_action_cb(wid, key_event)
+        #now fallback to default behaviour:
+        KeyboardBase.process_key_event(self, send_key_action_cb, wid, key_event)
