@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,6 +15,12 @@ from xpra.util import AdHocStruct
 def warn(msg):
     sys.stderr.write(msg+"\n")
 
+def debug(*args):
+    #can be overriden
+    pass
+
+DEFAULT_XPRA_CONF_FILENAME = os.environ.get("XPRA_CONF_FILENAME", 'xpra.conf')
+
 
 ENCRYPTION_CIPHERS = []
 try:
@@ -41,17 +47,17 @@ def OpenGL_safety_check():
         try:
             rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
             if rnum<=[12, 4]:
-                return "Ubuntu %s is too buggy" % ur
+                return False, "Ubuntu %s is too buggy" % ur
         except:
             pass
     #try to detect VirtualBox:
     #based on the code found here:
     #http://spth.virii.lu/eof2/articles/WarGame/vboxdetect.html
-    #because it causes hard VM crashes when we probe the GL driver!
+    #because it used to cause hard VM crashes when we probe the GL driver!
     try:
         from ctypes import cdll
         if cdll.LoadLibrary("VBoxHook.dll"):
-            return "VirtualBox is present (VBoxHook.dll)"
+            return True, "VirtualBox is present (VBoxHook.dll)"
     except:
         pass
     try:
@@ -61,14 +67,14 @@ def OpenGL_safety_check():
         finally:
             if f:
                 f.close()
-                return "VirtualBox is present (VBoxMiniRdrDN)"
+                return True, "VirtualBox is present (VBoxMiniRdrDN)"
     except Exception, e:
         import errno
         if e.args[0]==errno.EACCES:
-            return "VirtualBox is present (VBoxMiniRdrDN)"
-    return None
+            return True, "VirtualBox is present (VBoxMiniRdrDN)"
+    return True, None
 OPENGL_DEFAULT = None       #will auto-detect by probing
-if OpenGL_safety_check() is not None:
+if not OpenGL_safety_check()[0]:
     OPENGL_DEFAULT = False
 
 
@@ -77,11 +83,7 @@ if OpenGL_safety_check() is not None:
 def get_build_info():
     info = []
     try:
-        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
-        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_VERSION
-        info.append("Built on %s by %s" % (BUILT_ON, BUILT_BY))
-        if BUILD_DATE:
-            info.append(BUILD_DATE)
+        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS             #@UnresolvedImport
         try:
             mods = int(LOCAL_MODIFICATIONS)
         except:
@@ -90,6 +92,13 @@ def get_build_info():
             info.append("revision %s" % REVISION)
         else:
             info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
+    except Exception, e:
+        warn("Error: could not find the source information: %s" % e)
+    try:
+        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_VERSION    #@UnresolvedImport
+        info.append("Built on %s by %s" % (BUILT_ON, BUILT_BY))
+        if BUILD_DATE:
+            info.append(BUILD_DATE)
         if CYTHON_VERSION!="unknown" or COMPILER_VERSION!="unknown":
             info.append("")
         if CYTHON_VERSION!="unknown":
@@ -97,19 +106,22 @@ def get_build_info():
         if COMPILER_VERSION!="unknown":
             info.append(COMPILER_VERSION)
     except Exception, e:
-        warn("Error: could not find the source or build information: %s" % e)
+        warn("Error: could not find the build information: %s" % e)
     return info
 
 
-def save_config(conf_file, config, keys, extras={}):
+def save_config(conf_file, config, keys, extras_types={}):
     f = open(conf_file, "w")
     option_types = OPTION_TYPES.copy()
-    option_types.update(extras)
+    option_types.update(extras_types)
+    saved = {}
     for key in keys:
         assert key in option_types, "invalid configuration key: %s" % key
         fn = key.replace("-", "_")
         v = getattr(config, fn)
+        saved[key] = v
         f.write("%s=%s%s" % (key, v, os.linesep))
+    debug("save_config: saved %s to %s", saved, conf_file)
     f.close()
 
 def read_config(conf_file):
@@ -120,18 +132,25 @@ def read_config(conf_file):
     """
     d = {}
     if not os.path.isfile(conf_file):
+        debug("read_config(%s) is not a file!", conf_file)
         return d
     f = open(conf_file, "rU")
     lines = []
+    no = 0
     for line in f:
         sline = line.strip().rstrip('\r\n').strip()
+        no += 1
         if len(sline) == 0:
+            debug("%4s empty line", no)
             continue
         if sline[0] in ( '!', '#' ):
+            debug("%4s skipping comments   : %s", no, sline[:16]+"..")
             continue
+        debug("%4s loaded              : %s", no, sline)
         lines.append(sline)
     f.close()
-    #aggregate any lines with trailing bacakslash
+    debug("loaded %s lines", len(lines))
+    #aggregate any lines with trailing backslash
     agg_lines = []
     l = ""
     for line in lines:
@@ -144,9 +163,11 @@ def read_config(conf_file):
     if len(l)>0:
         #last line had a trailing backslash... meh
         agg_lines.append(l)
+    debug("loaded %s aggregated lines", len(agg_lines))
     #parse name=value pairs:
     for sline in agg_lines:
         if sline.find("=")<=0:
+            debug("skipping line which is missing an equal sign: %s", sline)
             continue
         props = sline.split("=", 1)
         assert len(props)==2
@@ -158,58 +179,69 @@ def read_config(conf_file):
                 d[name] = current_value + [value]
             else:
                 d[name] = [current_value, value]
+            debug("added to: %s='%s'", name, d[name])
         else:
+            debug("assigned (new): %s='%s'", name, value)
             d[name] = value
     return  d
 
-def read_xpra_conf(conf_dir):
+def read_xpra_conf(conf_dir, xpra_conf_filename=DEFAULT_XPRA_CONF_FILENAME):
     """
-        Reads an "xpra.conf" file from the given directory,
+        Reads an <xpra_conf_filename> file from the given directory,
         returns a dict with values as strings and arrays of strings.
     """
+    debug("read_xpra_conf(%s, %s)", conf_dir, xpra_conf_filename)
     cdir = os.path.expanduser(conf_dir)
     d = {}
     if not os.path.exists(cdir) or not os.path.isdir(cdir):
+        debug("invalid config directory: %s", cdir)
         return  d
-    conf_file = os.path.join(cdir, 'xpra.conf')
+    conf_file = os.path.join(cdir, xpra_conf_filename)
     if not os.path.exists(conf_file) or not os.path.isfile(conf_file):
+        debug("config file does not exist: %s", conf_file)
         return  d
     return read_config(conf_file)
 
 def read_xpra_defaults():
     """
-        Reads the global "xpra.conf" and then the user-specific one.
+        Reads the global <xpra_conf_filename> from the <conf_dir>
+        and then the user-specific one.
         (the latter overrides values from the former)
         returns a dict with values as strings and arrays of strings.
+        If the <conf_dir> is not specified, we figure out its location.
     """
-    #first, read the global defaults:
-    from xpra.platform.paths import get_resources_dir, get_default_conf_dir
-    if sys.platform.startswith("win") or sys.platform.startswith("darwin"):
-        #OSX and win32 use binary installers,
-        #we must look for the default config in the bundled resource location:
-        conf_dir = get_resources_dir()
-    elif sys.prefix == '/usr':
-        #default posix config location:
-        conf_dir = '/etc/xpra'
-    else:
-        #hope the prefix is something like "/usr/local":
-        conf_dir = sys.prefix + '/etc/xpra/'
+    from xpra.platform.paths import get_default_conf_dir, get_system_conf_dir, get_user_conf_dir
+    # load config files in this order (the later ones override earlier ones):
+    # * application defaults   (ie: "/Volumes/Xpra/Xpra.app/Contents/Resources/" on OSX)
+    #                          (ie: "C:\Program Files\Xpra\" on win32)
+    #                          (ie: None on others)
+    # * system defaults        (ie: "/etc/xpra" on Posix - not on OSX)
+    #                          (ie: "/Library/Application Support/Xpra" on OSX)
+    #                          (ie: "C:\Documents and Settings\All Users\Application Data\Xpra" with XP)
+    #                          (ie: "C:\ProgramData\Xpra" with Vista onwards)
+    # * user config            (ie: "~/.xpra/" on all Posix, including OSX)
+    #                          (ie: "C:\Documents and Settings\Username\Application Data\Xpra" with XP)
+    #                          (ie: "C:\Users\<user name>\AppData\Roaming" with Visa onwards)
+    dirs = [get_default_conf_dir(), get_system_conf_dir(), get_user_conf_dir()]
     defaults = {}
-    if conf_dir:
-        defaults = read_xpra_conf(conf_dir)
-    #now load the per-user config over it:
-    def_dir = get_default_conf_dir()
-    if def_dir and def_dir!=conf_dir:
-        user_defaults = read_xpra_conf(get_default_conf_dir())
-        defaults.update(user_defaults)
+    for d in dirs:
+        if not d:
+            continue
+        ad = os.path.expanduser(d)
+        if not os.path.exists(ad):
+            debug("read_xpra_defaults: skipping %s", ad)
+            continue
+        defaults.update(read_xpra_conf(ad))
+        debug("read_xpra_defaults: updated defaults with %s", ad)
     return defaults
 
 
+OPTIONS_VALIDATION = {}
+
 OPTION_TYPES = {
                     #string options:
                     "encoding"          : str,
                     "title"             : str,
-                    "host"              : str,
                     "username"          : str,
                     "auth"              : str,
                     "remote-xpra"       : str,
@@ -218,7 +250,6 @@ OPTION_TYPES = {
                     "dock-icon"         : str,
                     "tray-icon"         : str,
                     "window-icon"       : str,
-                    "password"          : str,
                     "password-file"     : str,
                     "clipboard-filter-file" : str,
                     "pulseaudio-command": str,
@@ -229,18 +260,17 @@ OPTION_TYPES = {
                     "xvfb"              : str,
                     "socket-dir"        : str,
                     "log-file"          : str,
-                    "mode"              : str,
                     "border"            : str,
                     "window-layout"     : str,
                     "display"           : str,
                     "tcp-proxy"         : str,
                     "debug"             : str,
+                    "input-method"      : str,
                     #int options:
                     "quality"           : int,
                     "min-quality"       : int,
                     "speed"             : int,
                     "min-speed"         : int,
-                    "port"              : int,
                     "compression_level" : int,
                     "dpi"               : int,
                     #float options:
@@ -249,6 +279,7 @@ OPTION_TYPES = {
                     #boolean options:
                     "daemon"            : bool,
                     "use-display"       : bool,
+                    "displayfd"         : bool,
                     "fake-xinerama"     : bool,
                     "tray"              : bool,
                     "clipboard"         : bool,
@@ -269,18 +300,21 @@ OPTION_TYPES = {
                     "sharing"           : bool,
                     "delay-tray"        : bool,
                     "windows"           : bool,
-                    "autoconnect"       : bool,
                     "exit-with-children": bool,
                     "exit-with-client"  : bool,
                     "exit-ssh"          : bool,
                     "opengl"            : bool,
                     "mdns"              : bool,
                     "swap-keys"         : bool,
-                    #arrays of strings (default value, allowed options):
+                    #arrays of strings:
+                    "encodings"         : list,
                     "video-encoders"    : list,
                     "csc-modules"       : list,
+                    "video-decoders"    : list,
                     "speaker-codec"     : list,
                     "microphone-codec"  : list,
+                    "compressors"       : list,
+                    "packet-encoders"   : list,
                     "key-shortcut"      : list,
                     "start-child"       : list,
                     "bind-tcp"          : list,
@@ -293,7 +327,7 @@ def get_defaults():
     global GLOBAL_DEFAULTS
     if GLOBAL_DEFAULTS is not None:
         return GLOBAL_DEFAULTS
-    from xpra.platform.features import DEFAULT_SSH_CMD
+    from xpra.platform.features import DEFAULT_SSH_CMD, DEFAULT_PULSEAUDIO_COMMAND, DEFAULT_XVFB_COMMAND
     try:
         from xpra.platform.info import get_username
         username = get_username()
@@ -302,7 +336,6 @@ def get_defaults():
     GLOBAL_DEFAULTS = {
                     "encoding"          : "",
                     "title"             : "@title@ on @client-machine@",
-                    "host"              : "",
                     "username"          : username,
                     "auth"              : "",
                     "remote-xpra"       : "~/.xpra/run-xpra",
@@ -311,18 +344,13 @@ def get_defaults():
                     "dock-icon"         : "",
                     "tray-icon"         : "",
                     "window-icon"       : "",
-                    "password"          : "",
                     "password-file"     : "",
                     "clipboard-filter-file" : "",
-                    "pulseaudio-command": "pulseaudio --start --daemonize=false --system=false "
-                                            +" --exit-idle-time=-1 -n --load=module-suspend-on-idle "
-                                            +" --load=module-null-sink --load=module-native-protocol-unix "
-                                            +" --log-level=2 --log-target=stderr",
+                    "pulseaudio-command": DEFAULT_PULSEAUDIO_COMMAND,
                     "encryption"        : "",
                     "encryption_keyfile": "",
-                    "mode"              : "tcp",
                     "ssh"               : DEFAULT_SSH_CMD,
-                    "xvfb"              : "Xvfb +extension Composite -screen 0 3840x2560x24+32 -nolisten tcp -noreset -auth $XAUTHORITY",
+                    "xvfb"              : DEFAULT_XVFB_COMMAND,
                     "socket-dir"        : "",
                     "log-file"          : "$DISPLAY.log",
                     "border"            : "auto,0",
@@ -330,17 +358,18 @@ def get_defaults():
                     "display"           : "",
                     "tcp-proxy"         : "",
                     "debug"             : "",
-                    "quality"           : -1,
-                    "min-quality"       : 50,
-                    "speed"             : -1,
-                    "min-speed"         : -1,
-                    "port"              : -1,
+                    "input-method"      : "none",
+                    "quality"           : 0,
+                    "min-quality"       : 30,
+                    "speed"             : 0,
+                    "min-speed"         : 0,
                     "compression_level" : 1,
                     "dpi"               : 96,
                     "max-bandwidth"     : 0.0,
                     "auto-refresh-delay": 0.25,
                     "daemon"            : True,
                     "use-display"       : False,
+                    "displayfd"         : False,
                     "fake-xinerama"     : True,
                     "tray"              : True,
                     "clipboard"         : True,
@@ -361,30 +390,25 @@ def get_defaults():
                     "sharing"           : False,
                     "delay-tray"        : False,
                     "windows"           : True,
-                    "autoconnect"       : False,
                     "exit-with-children": False,
                     "exit-with-client"  : False,
                     "exit-ssh"          : True,
                     "opengl"            : OPENGL_DEFAULT,
                     "mdns"              : os.name=="posix" and not sys.platform.startswith("darwin"),
                     "swap-keys"         : sys.platform.startswith("darwin"),    #only used on osx
-                    "video-encoders"    : [],
-                    "csc-modules"       : [],
+                    "encodings"         : ["all"],
+                    "video-encoders"    : ["all"],
+                    "csc-modules"       : ["all"],
+                    "video-decoders"    : ["all"],
                     "speaker-codec"     : [],
                     "microphone-codec"  : [],
+                    "compressors"       : ["all"],
+                    "packet-encoders"   : ["all"],
                     "key-shortcut"      : ["Meta+Shift+F4:quit", "Meta+Shift+F8:magic_key", "Meta+Shift+F11:show_session_info"],
                     "bind-tcp"          : None,
                     "start-child"       : None,
                     }
     return GLOBAL_DEFAULTS
-MODES = ["tcp", "tcp + aes", "ssh"]
-def validate_in_list(x, options):
-    if x in options:
-        return None
-    return "must be in %s" % (", ".join(options))
-OPTIONS_VALIDATION = {
-                    "mode"              : lambda x : validate_in_list(x, MODES),
-                    }
 #fields that got renamed:
 CLONES = {
             "quality"       : "jpeg-quality",
@@ -417,7 +441,7 @@ def print_bool(k, v, true_str='yes', false_str='no'):
         return false_str
     warn("Warning: cannot print value '%s' for '%s' as a boolean" % (v, k))
 
-def parse_number(numtype, k, v, auto=-1):
+def parse_number(numtype, k, v, auto=0):
     if type(v)==str:
         v = v.lower()
     if v=="auto":
@@ -428,15 +452,17 @@ def parse_number(numtype, k, v, auto=-1):
         warn("Warning: cannot parse value '%s' for '%s' as a type %s: %s" % (v, k, numtype, e))
         return None
 
-def validate_config(d={}, discard=NO_FILE_OPTIONS, extras={}):
+def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_validation={}):
     """
         Validates all the options given in a dict with fields as keys and
         strings or arrays of strings as values.
         Each option is strongly typed and invalid value are discarded.
         We get the required datatype from OPTION_TYPES
     """
+    validations = OPTIONS_VALIDATION.copy()
+    validations.update(extras_validation)
     option_types = OPTION_TYPES.copy()
-    option_types.update(extras)
+    option_types.update(extras_types)
     nd = {}
     for k, v in d.items():
         if k in discard:
@@ -474,7 +500,7 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras={}):
                 continue
         else:
             warn("Error: unknown option type for '%s': %s" % (k, vt))
-        validation = OPTIONS_VALIDATION.get(k)
+        validation = validations.get(k)
         if validation and v is not None:
             msg = validation(v)
             if msg:
@@ -484,11 +510,16 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras={}):
     return nd
 
 
-def make_defaults_struct():
+def make_defaults_struct(extras_defaults={}, extras_types={}, extras_validation={}):
     #populate config with default values:
     defaults = read_xpra_defaults()
-    validated = validate_config(defaults)
+    return dict_to_validated_config(defaults, extras_defaults, extras_types, extras_validation)
+
+def dict_to_validated_config(d, extras_defaults={}, extras_types={}, extras_validation={}):
     options = get_defaults().copy()
+    options.update(extras_defaults)
+    #parse config:
+    validated = validate_config(d, extras_types=extras_types, extras_validation=extras_validation)
     options.update(validated)
     for k,v in CLONES.items():
         if k in options:
@@ -501,7 +532,40 @@ def make_defaults_struct():
 
 
 def main():
-    print("default configuration: %s" % make_defaults_struct())
+    from xpra.util import nonl
+    def print_options(o):
+        for k,ot in sorted(OPTION_TYPES.items()):
+            attr_name = k.replace("-", "_")
+            v = getattr(o, attr_name, "")
+            if ot==bool and v is None:
+                v = "Auto"
+            if type(v)==list:
+                v = ", ".join(str(x) for x in v)
+            print("* %-32s : %s" % (k, nonl(v)))
+    from xpra.platform import init, clean
+    try:
+        init("Config-Info", "Config Info")
+        args = list(sys.argv[1:])
+        if "-v" in args:
+            global debug
+            def debug(*args):
+                print(args[0] % args[1:])
+            args.remove("-v")
+
+        print("Default Configuration:")
+        print_options(make_defaults_struct())
+        if len(args)>0:
+            for filename in args:
+                print("")
+                print("Configuration file '%s':" % filename)
+                if not os.path.exists(filename):
+                    print(" Error: file not found")
+                    continue
+                d = read_config(filename)
+                config = dict_to_validated_config(d)
+                print_options(config)
+    finally:
+        clean()
 
 
 if __name__ == "__main__":
