@@ -201,6 +201,10 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
         #XPRA_SOCKET_DIR is a special case, it is handled below
         if var=="XPRA_SOCKET_DIR":
             continue
+        if var.startswith("BASH_FUNC"):
+            #some versions of bash will apparently generate functions
+            #that cannot be reloaded using this script
+            continue
         # :-separated envvars that people might change while their server is
         # going:
         if var in ("PATH", "LD_LIBRARY_PATH", "PYTHONPATH"):
@@ -824,9 +828,13 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
         logfd = open_log_file(log_filename0)
         assert logfd > 2
         stdout, stderr = daemonize(logfd)
-        stderr.write("Entering daemon mode; "
+        try:
+            stderr.write("Entering daemon mode; "
                  + "any further errors will be reported to:\n"
                  + ("  %s\n" % log_filename0))
+        except:
+            #this can happen if stderr is closed by the caller already
+            pass
 
     # Write out a shell-script so that we can start our proxy in a clean
     # environment:
