@@ -50,7 +50,7 @@ def calculate_batch_delay(wid, window_dimensions, has_focus, other_is_fullscreen
     factors.append(("maximized", {"other_is_maximized" : other_is_maximized}, 4*int(other_is_maximized), int(other_is_maximized)))
     #soft expired regions is a strong indicator of problems:
     #(0 for none, up to max_soft_expired which is 5)
-    factors.append(("soft-expired", {"soft_expired" : soft_expired}, soft_expired, int(bool(soft_expired))))
+    factors.append(("soft-expired", {"count" : soft_expired}, soft_expired, int(bool(soft_expired))))
     #now use those factors to drive the delay change:
     update_batch_delay(batch, factors)
 
@@ -109,12 +109,9 @@ def get_target_speed(wid, window_dimensions, batch, global_statistics, statistic
 
     #megapixels per second:
     mpixels = low_limit/1024.0/1024.0
-    if mpixels>1:
-        #for larger window sizes, we should be downscaling,
-        #and don't want to wait too long for those anyway:
-        ref_damage_latency = 0.010 + 0.030 * (1+mathlog(mpixels))
-    else:
-        ref_damage_latency = 0.010 + 0.030 * mpixels
+    #for larger window sizes, we should be downscaling,
+    #and don't want to wait too long for those anyway:
+    ref_damage_latency = 0.010 + 0.025 * (1+mathlog(max(1, mpixels)))
 
     #abs: try to never go higher than 5 times reference latency:
     dam_lat_abs = max(0, ((statistics.avg_damage_in_latency or 0)-ref_damage_latency) / (ref_damage_latency * 4.0))
@@ -185,7 +182,8 @@ def get_target_quality(wid, window_dimensions, batch, global_statistics, statist
             #so when we start and we don't have any records, we don't lower quality
             #just because the start delay is higher than min_delay
             ref_delay = (batch.START_DELAY*10.0/recs + batch.min_delay*recs) / (recs+10.0/recs)
-            batch_q = ref_delay / max(batch.min_delay, batch.delay)
+            #anything less than twice the minimum is good enough:
+            batch_q = ref_delay / max(1, batch.min_delay, batch.delay/2.0)
             target = min(1.0, target, batch_q)
     latency_q = -1
     if len(global_statistics.client_latency)>0 and global_statistics.recent_client_latency>0:
@@ -197,7 +195,9 @@ def get_target_quality(wid, window_dimensions, batch, global_statistics, statist
     info = {
             "min_quality"   : min_quality,
             "backlog_factor": int(100.0*packets_bl),
-            "batch_factor"  : int(100.0*batch_q),
-            "latency_factor": int(100.0*latency_q),
             }
+    if batch_q>=0:
+        info["batch_factor"] = int(100.0*batch_q)
+    if latency_q>=0:
+        info["latency_factor"] = int(100.0*latency_q)
     return info, target_quality
