@@ -16,8 +16,9 @@ log = Logger("server", "gtk")
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable)
 from xpra.server.server_base import ServerBase
-from xpra.gtk_common.gtk_util import add_gtk_version_info
+from xpra.gtk_common.gtk_util import get_gtk_version_info
 from xpra.gtk_common.gtk2common import gtk2main
+from xpra.util import updict
 
 
 class GTKServerBase(ServerBase):
@@ -40,37 +41,46 @@ class GTKServerBase(ServerBase):
         gtk.gdk.keymap_get_default().connect("keys-changed", self._keys_changed)
 
     def do_quit(self):
+        log("do_quit: calling gtk_main_quit_really")
         gtk_main_quit_really()
+        log("do_quit: gtk_main_quit_really done")
 
     def do_run(self):
         gtk_main_quit_on_fatal_exceptions_enable()
+        log("do_run() calling %s", gtk2main)
         gtk2main()
-        log("end of gtk.main()")
+        log("do_run() end of gtk.main()")
 
     def add_listen_socket(self, socktype, sock):
         sock.listen(5)
         gobject.io_add_watch(sock, gobject.IO_IN, self._new_connection, sock)
         self.socket_types[sock] = socktype
 
-    def make_hello(self):
-        capabilities = ServerBase.make_hello(self)
-        capabilities["display"] = gtk.gdk.display_get_default().get_name()
-        capabilities["cursor.default_size"] = gtk.gdk.display_get_default().get_default_cursor_size()
-        capabilities["cursor.max_size"] = gtk.gdk.display_get_default().get_maximal_cursor_size()
-        add_gtk_version_info(capabilities, gtk, "", True)
+    def make_hello(self, source):
+        capabilities = ServerBase.make_hello(self, source)
+        if source.wants_display:
+            display = gtk.gdk.display_get_default()
+            capabilities.update({
+                "display"               : display.get_name(),
+                "cursor.default_size"   : display.get_default_cursor_size(),
+                "cursor.max_size"       : display.get_maximal_cursor_size()})
+        if source.wants_versions:
+            capabilities.update(get_gtk_version_info())
         return capabilities
 
     def get_ui_info(self, *args):
         info = ServerBase.get_ui_info(self, *args)
+        info["server.display"] = gtk.gdk.display_get_default().get_name()
         info["server.root_window_size"] = self.get_root_window_size()
         return info
 
     def do_get_info(self, proto, *args):
         info = ServerBase.do_get_info(self, proto, *args)
         ss = self._server_sources.get(proto)
-        add_gtk_version_info(info, gtk, "server.", (ss is None) or ss.namespace)
-        info["server.type"] = "Python/gtk-x11"
-        info["features.randr"] = self.randr
+        updict(info, "server", get_gtk_version_info(new_namespace=(ss is not None) and ss.namespace))
+        info.update({
+                     "server.type"      : "Python/gtk-x11",
+                     "features.randr"   : self.randr})
         return info
 
     def get_root_window_size(self):
@@ -111,6 +121,12 @@ class GTKServerBase(ServerBase):
     def set_workarea(self, workarea):
         pass
 
+    def set_desktop_geometry(self, width, height):
+        pass
+
+    def set_dpi(self, xdpi, ydpi):
+        pass
+
 
     def _move_pointer(self, wid, pos):
         x, y = pos
