@@ -16,6 +16,7 @@ try:
 except Exception, e:
     log("cannot load argb module: %s", e)
     bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba = (None,)*4
+from xpra.codecs.codec_constants import get_PIL_encodings
 from xpra.os_util import StringIOClass
 from xpra.codecs.loader import get_codec, get_codec_version
 from xpra.os_util import builtins
@@ -39,10 +40,18 @@ def warn_encoding_once(key, message):
         encoding_warnings.add(key)
 
 
-def webp_encode(coding, image, supports_transparency, quality, speed, options):
+def webp_encode(coding, image, rgb_formats, supports_transparency, quality, speed, options):
+    pixel_format = image.get_pixel_format()
+    #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4), pixel_format, rgb_formats)
+    if pixel_format not in rgb_formats:
+        if not rgb_reformat(image, rgb_formats, supports_transparency):
+            raise Exception("cannot find compatible rgb format to use for %s! (supported: %s)" % (pixel_format, rgb_formats))
+        #get the new format:
+        pixel_format = image.get_pixel_format()
     stride = image.get_rowstride()
     enc_webp = get_codec("enc_webp")
-    if enc_webp and stride>0 and stride%4==0 and image.get_pixel_format() in ("BGRA", "BGRX"):
+    #log("webp_encode%s stride=%s, enc_webp=%s", (coding, image, rgb_formats, supports_transparency, quality, speed, options), stride, enc_webp)
+    if enc_webp and stride>0 and stride%4==0 and image.get_pixel_format() in ("BGRA", "BGRX", "RGBA", "RGBX"):
         #prefer Cython module:
         alpha = supports_transparency and image.get_pixel_format().find("A")>=0
         w = image.get_width()
@@ -56,14 +65,20 @@ def webp_encode(coding, image, supports_transparency, quality, speed, options):
             speed = int(sqrt(speed) * 10)
         speed = max(0, min(100, speed))
         cdata = enc_webp.compress(image.get_pixels(), w, h, stride=stride/4, quality=quality, speed=speed, has_alpha=alpha)
-        client_options = {"speed" : speed}
+        client_options = {"speed"       : speed,
+                          "rgb_format"  : pixel_format}
         if quality>=0 and quality<=100:
             client_options["quality"] = quality
         if alpha:
             client_options["has_alpha"] = True
         return "webp", compression.Compressed("webp", cdata), client_options, image.get_width(), image.get_height(), 0, 24
     #fallback to PIL
-    return PIL_encode(coding, image, quality, speed, supports_transparency)
+    log("using PIL fallback for webp: enc_webp=%s, stride=%s, pixel format=%s", enc_webp, stride, image.get_pixel_format())
+    encs = get_PIL_encodings(PIL)
+    for x in ("webp", "png"):
+        if x in encs:
+            return PIL_encode(coding, image, quality, speed, supports_transparency)
+    raise Exception("BUG: cannot use 'webp' encoding and none of the PIL fallbacks are available!")
 
 
 def roundup(n, m):
