@@ -184,18 +184,18 @@ class ServerBase(ServerCore):
         #Pithon Imaging Libary:
         PIL = get_codec("PIL")
         if PIL:
-            add_encodings(get_PIL_encodings(PIL))
+            pil_encs = get_PIL_encodings(PIL)
+            add_encodings(pil_encs)
+            #Note: webp will only be enabled if we have a Python-PIL fallback
+            #(either "webp" or "png")
+            if has_codec("enc_webp") and ("webp" in pil_encs or "png" in pil_encs):
+                add_encodings(["webp"])
+                self.lossless_mode_encodings.append("webp")
         #now update the variables:
         self.encodings = encs
         self.core_encodings = core_encs
-        #Note: webp will only be enabled if PIL supports it,
-        #that's because "enc_webp" needs a fallback for odd strides and pixel formats
-        #(probably never gets used but difficult to predict for sure)
-        self.lossless_encodings = [x for x in self.core_encodings if (x.startswith("png") or x.startswith("rgb"))]
+        self.lossless_encodings = [x for x in self.core_encodings if (x.startswith("png") or x.startswith("rgb") or x=="webp")]
         self.lossless_mode_encodings = []
-        if has_codec("enc_webp"):
-            self.lossless_mode_encodings.append("webp")
-            self.lossless_encodings.append("webp")
         pref = [x for x in PREFERED_ENCODING_ORDER if x in self.encodings]
         if pref:
             self.default_encoding = pref[0]
@@ -309,7 +309,7 @@ class ServerBase(ServerCore):
         self.keys_pressed = {}
         self.keys_timedout = {}
         #timers for cancelling key repeat when we get jitter
-        self.keys_repeat_timers = {}
+        self.key_repeat_timer = None
         self.watch_keymap_changes()
 
     def watch_keymap_changes(self):
@@ -981,7 +981,6 @@ class ServerBase(ServerCore):
                     source.send_client_command(*client_command)
                 else:
                     commandlog.warn("client %s does not support client command %s", source, client_command[0])
-            csource.send_client_command(*client_command)
             return 0, "client control command '%s' forwarded to %s clients" % (client_command[0], count)
         else:
             return ServerCore.do_handle_command_request(self, command, args)
@@ -1550,24 +1549,25 @@ class ServerBase(ServerCore):
         if not is_mod and self.keyboard_sync and self.key_repeat_delay>0 and self.key_repeat_interval>0:
             self._key_repeat(wid, pressed, name, keyval, keycode, modifiers, self.key_repeat_delay)
 
+    def cancel_key_repeat_timer(self):
+        if self.key_repeat_timer:
+            self.source_remove(self.key_repeat_timer)
+            self.key_repeat_timer = None
+
     def _key_repeat(self, wid, pressed, keyname, keyval, keycode, modifiers, delay_ms=0):
         """ Schedules/cancels the key repeat timeouts """
-        timer = self.keys_repeat_timers.get(keycode, None)
-        if timer:
-            del self.keys_repeat_timers[keycode]
-            keylog("cancelling key repeat timer: %s for %s / %s", timer, keyname, keycode)
-            self.source_remove(timer)
+        self.cancel_key_repeat_timer()
         if pressed:
             delay_ms = min(1500, max(250, delay_ms))
             keylog("scheduling key repeat timer with delay %s for %s / %s", delay_ms, keyname, keycode)
             def _key_repeat_timeout(when):
-                del self.keys_repeat_timers[keycode]
+                self.key_repeat_timer = None
                 now = time.time()
                 keylog("key repeat timeout for %s / '%s' - clearing it, now=%s, scheduled at %s with delay=%s", keyname, keycode, now, when, delay_ms)
                 self._handle_key(wid, False, keyname, keyval, keycode, modifiers)
                 self.keys_timedout[keycode] = now
             now = time.time()
-            self.keys_repeat_timers[keycode] = self.timeout_add(delay_ms, _key_repeat_timeout, now)
+            self.key_repeat_timer = self.timeout_add(delay_ms, _key_repeat_timeout, now)
 
     def _process_key_repeat(self, proto, packet):
         wid, keyname, keyval, client_keycode, modifiers = packet[1:6]
