@@ -21,7 +21,6 @@ cursorlog = Logger("cursor")
 
 from xpra.server import ClientException
 from xpra.server.source_stats import GlobalPerformanceStatistics
-from xpra.server.window_source import WindowSource, HAS_ALPHA
 from xpra.server.window_video_source import WindowVideoSource
 from xpra.server.batch_config import DamageBatchConfig
 from xpra.simple_stats import add_list_stats, std_unit
@@ -544,7 +543,6 @@ class ServerSource(object):
         self.notify_startup_complete = c.boolget("notify-startup-complete")
         self.namespace = c.boolget("namespace")
         self.control_commands = c.strlistget("control_commands")
-        self.supports_transparency = HAS_ALPHA and c.boolget("encoding.transparency")
 
         self.desktop_size = c.intpair("desktop_size")
         if self.desktop_size is not None:
@@ -743,7 +741,7 @@ class ServerSource(object):
     def new_sound_buffer(self, sound_source, data, metadata):
         soundlog("new_sound_buffer(%s, %s, %s) source=%s, suspended=%s, sequence=%s",
                  sound_source, len(data or []), metadata, self.sound_source, self.suspended, self.sound_source_sequence)
-        if self.sound_source is None:
+        if self.sound_source is None or self.is_closed():
             return
         if self.sound_source_sequence>0:
             metadata["sequence"] = self.sound_source_sequence
@@ -827,6 +825,8 @@ class ServerSource(object):
 
     def sound_data(self, codec, data, metadata, *args):
         soundlog("sound_data(%s, %s, %s, %s) sound sink=%s", codec, len(data or []), metadata, args, self.sound_sink)
+        if self.is_closed():
+            return
         if self.sound_sink is not None and codec!=self.sound_sink.codec:
             log.info("sound codec changed from %s to %s", self.sound_sink.codec, codec)
             self.sound_sink.cleanup()
@@ -1518,15 +1518,7 @@ class ServerSource(object):
         ws = self.window_sources.get(wid)
         if ws is None:
             batch_config = self.make_batch_config(wid, window)
-            wclass = WindowSource
-            #don't use video for system trays
-            #or for transparent windows (if the client supports transparency) since video doesn't do alpha (yet?)
-            if not window.is_tray() and (not window.has_alpha() or not self.supports_transparency):
-                wts = get_generic_window_type(window, strip_net=True)
-                if set(("NORMAL", "DIALOG", "DESKTOP")).intersection(wts):
-                    wclass = WindowVideoSource
-
-            ws = wclass(self.idle_add, self.timeout_add, self.source_remove,
+            ws = WindowVideoSource(self.idle_add, self.timeout_add, self.source_remove,
                               self.queue_size, self.queue_damage, self.queue_packet,
                               self.statistics,
                               wid, window, batch_config, self.auto_refresh_delay,
