@@ -203,6 +203,7 @@ class WindowSource(object):
         self.auto_refresh_delay = 0
         self.video_helper = None
         self.refresh_event_time = 0
+        self.refresh_target_time = 0
         self.refresh_timer = None
         self.refresh_regions = []
         self.timeout_timer = None
@@ -261,8 +262,6 @@ class WindowSource(object):
         info = {
                 "dimensions"            : self.window_dimensions,
                 "encoding"              : self.encoding,
-                "encoding.mmap"         : bool(self._mmap) and (self._mmap_size>0),
-                "encoding.last_used"    : self.encoding_last_used or "",
                 "suspended"             : self.suspended or False
                 }
         def up(prefix, d):
@@ -272,11 +271,26 @@ class WindowSource(object):
         up("encoding.lossless_threshold", {
                 "base"                  : self._lossless_threshold_base,
                 "pixel_boost"           : self._lossless_threshold_pixel_boost})
-        info["encoding.rgb_threshold"] = self._rgb_auto_threshold
-
+        up("encoding", {
+                "rgb_threshold"         : self._rgb_auto_threshold,
+                "mmap"                  : bool(self._mmap) and (self._mmap_size>0),
+                "last_used"             : self.encoding_last_used or "",
+                "full-frames-only"      : self.full_frames_only,
+                "supports-transparency" : self.supports_transparency,
+                })
+        up("encoding",  self.get_quality_speed_info())
+        try:
+            info["encoding.selection"] = self.get_best_encoding.__name__
+        except:
+            pass
         up("property",  self.get_property_info())
         up("batch",     self.batch_config.get_info())
-        up("encoding",  self.get_quality_speed_info())
+        up("encodings", {
+                 ""                     : self.encodings,
+                 "core"                 : self.core_encodings,
+                 "auto-refresh"         : self.client_refresh_encodings,
+                 "rgb_formats"          : self.rgb_formats,
+                 })
         info.update(self.statistics.get_info())
         return info
 
@@ -384,6 +398,9 @@ class WindowSource(object):
         #now we have the real list of encodings we can use:
         #"rgb32" and "rgb24" encodings are both aliased to "rgb"
         common_encodings = [x for x in self._encoders.keys() if x in self.core_encodings]
+        #"rgb" is a pseudo encoding and needs special code:
+        if "rgb24" in  common_encodings or "rgb32" in common_encodings:
+            common_encodings.append("rgb")
         if self.webp_leaks and "webp" in common_encodings:
             common_encodings.remove("webp")
         self.common_encodings = [x for x in PREFERED_ENCODING_ORDER if x in common_encodings]
@@ -439,6 +456,12 @@ class WindowSource(object):
                 return self.get_strict_encoding
             #choose an alpha encoding and keep it?
             return self.get_transparent_encoding
+        elif self.encoding=="rgb":
+            #if we're here we don't need alpha, so try rgb24 first:
+            if "rgb24" in self.common_encodings:
+                return self.encoding_is_rgb24
+            elif "rgb32" in self.common_encodings:
+                return self.encoding_is_rgb32
         #stick to what is specified or use rgb for small regions:
         return self.get_current_or_rgb
 
@@ -521,6 +544,7 @@ class WindowSource(object):
             self.source_remove(self.refresh_timer)
             self.refresh_timer = None
             self.refresh_event_time = 0
+            self.refresh_target_time = 0
 
     def cancel_timeout_timer(self):
         if self.timeout_timer:
@@ -726,7 +750,15 @@ class WindowSource(object):
         self.expire_timer = self.timeout_add(delay, self.expire_delayed_region, delay)
 
     def must_batch(self, delay):
-        return FORCE_BATCH or self.batch_config.always or delay>self.batch_config.min_delay
+        if FORCE_BATCH or self.batch_config.always or delay>self.batch_config.min_delay:
+            return True
+        try:
+            t, _ = self.batch_config.last_delays[-5]
+            #do batch if we got more than 5 damage events in the last 10 milliseconds:
+            return time.time()-t<0.010
+        except:
+            #probably not enough events to grab -10
+            return False
 
 
     def expire_delayed_region(self, delay):
@@ -1026,22 +1058,44 @@ class WindowSource(object):
                     msg = "covered all regions that needed a refresh, cancelling refresh"
                     self.cancel_refresh_timer()
                 else:
-                    msg = "removed rectangle from regions"
+                    msg = "removed rectangle from regions, keeping refresh"
         else:
+            #if we're here: the window is still valid and this was a lossy update,
+            #of some form (lossy encoding with low enough quality, or using CSC subsampling, or using scaling)
+            #so we probably need an auto-refresh (re-schedule it if one was due already)
+
             #try to add the rectangle to the refresh list:
-            if not self.add_refresh_region(window, region):
-                msg = "list of refresh regions unchanged"
+            pixels_modified = self.add_refresh_region(window, region)
+            #the target time is only set in this function and cleared when the refresh runs,
+            #copy it before we modify anything (as the refresh clears it from another thread)
+            target_time = self.refresh_target_time
+            if pixels_modified==0:
+                msg = "keeping existing timer (all pixels outside area)"
+                if self.refresh_regions and not target_time:
+                    #this should never happen:
+                    refreshlog.warn("refresh regions are pending but no refresh timer is due!")
             else:
-                #if we're here: the window is still valid and this was a lossy update,
-                #of some form (lossy encoding with low enough quality, or using CSC subsampling, or using scaling)
-                #so we need an auto-refresh (re-schedule it if one was due already)
-                if self.refresh_event_time>0:
-                    msg = "keeping existing timer"
-                else:
-                    msg = "scheduling refresh"
+                #some pixels were modified, and we do need to refresh them:
+                now = time.time()
+                #figure out the proportion of pixels updated:
+                pixels = region.width*region.height
+                ww, wh = window.get_dimensions()
+                pct = 100*pixels/(ww*wh)
+                #important: must check both, I think:
+                if target_time==0 or not self.refresh_timer:
+                    #this means we must schedule the refresh
                     self.refresh_event_time = time.time()
-                    sched_delay = int(max(50, self.auto_refresh_delay, self.batch_config.delay*4))
-                    self.refresh_timer = self.timeout_add(sched_delay, self.schedule_auto_refresh, window, options)
+                    #delay in milliseconds: always at least the settings,
+                    #more if we have more than 50% of the window pixels to update:
+                    sched_delay = int(max(50, self.auto_refresh_delay * max(50, pct) / 50, self.batch_config.delay*4))
+                    self.refresh_target_time = now + sched_delay/1000.0
+                    self.refresh_timer = self.timeout_add(int(sched_delay), self.refresh_timer_function, window, options)
+                    msg = "scheduling refresh in %sms (pct=%s, batch=%s)" % (sched_delay, pct, self.batch_config.delay)
+                else:
+                    #add to the target time, but this will not move it forwards for small updates following big ones:
+                    sched_delay = int(max(50, self.auto_refresh_delay * pct / 50, self.batch_config.delay*2))
+                    self.refresh_target_time = max(target_time, now + sched_delay/1000.0)
+                    msg = "re-scheduling refresh (due in %ims, %ims added - sched_delay=%s, pct=%s, batch=%s)" % (1000*(self.refresh_target_time-now), 1000*(self.refresh_target_time-target_time), sched_delay, pct, self.batch_config.delay)
         refreshlog("auto refresh: %5s screen update (quality=%3i), %s (region=%s, refresh regions=%s)", encoding, actual_quality, msg, region, self.refresh_regions)
 
     def remove_refresh_region(self, region):
@@ -1051,10 +1105,11 @@ class WindowSource(object):
 
     def add_refresh_region(self, window, region):
         #adds the given region to the refresh list
-        #returns True if the list was modified
+        #returns the number of pixels in the region update
         #(overriden in window video source to exclude the video region)
         #Note: this does not run in the UI thread!
-        return add_rectangle(self.refresh_regions, region)
+        add_rectangle(self.refresh_regions, region)
+        return region.width*region.height
 
     def can_refresh(self, window):
         #safe to call from any thread (does not call X11):
@@ -1066,9 +1121,12 @@ class WindowSource(object):
             return False
         return True
 
-    def schedule_auto_refresh(self, window, damage_options):
+    def refresh_timer_function(self, window, damage_options):
         """ Must be called from the UI thread:
-            this makes it easier to prevent races
+            this makes it easier to prevent races and we're allowed to use the window object.
+            And for that reason, it may re-schedule itself safely here too.
+            We figure out if now is the right time to do the refresh,
+            and if not re-schedule.
         """
         #timer is running now, clear so we don't try to cancel it somewhere else:
         self.refresh_timer = None
@@ -1079,33 +1137,22 @@ class WindowSource(object):
         ret = self.refresh_event_time
         if ret==0:
             return
-
-        #decide if now is the right time, or if we delay some more
-        #(the more pixels we have to refresh, the longer we wait)
-        pixels = sum(r.width*r.height for r in self.refresh_regions)
-        ww, wh = window.get_dimensions()
-        pct = 100*pixels/(ww*wh)
-        #target auto_refresh_delay, but double that if we have a full screen update:
-        target_delay = max(50, self.auto_refresh_delay * pct / 50)
-        elapsed = int(1000.0*(time.time()-ret))
-        if elapsed>=(target_delay-20):
-            #close enough to target, do it now:
-            refreshlog("schedule_auto_refresh: elapsed time %i with target=%i, refreshing now", elapsed, target_delay)
+        delta = self.refresh_target_time - time.time()
+        if delta<0.050:
+            #this is about right (due already or due shortly)
             self.timer_full_refresh(window)
-        else:
-            #delay a bit more:
-            delay = int(max(20, self.auto_refresh_delay, target_delay - elapsed))
-            refreshlog("schedule_auto_refresh: rescheduling auto refresh timer with extra delay %i (%i%% of window, refresh delay=%i, target=%i, elapsed=%i)", delay, pct, self.auto_refresh_delay, target_delay, elapsed)
-            self.refresh_timer = self.timeout_add(delay, self.timer_full_refresh, window)
-        return False
+            return
+        #re-schedule ourselves:
+        self.refresh_timer = self.timeout_add(int(delta*1000), self.refresh_timer_function, window, damage_options)
+        refreshlog("refresh_timer_function: rescheduling auto refresh timer with extra delay %ims", int(1000*delta))
 
     def timer_full_refresh(self, window):
+        #copy event time and list of regions (which may get modified by another thread)
         ret = self.refresh_event_time
-        self.refresh_timer = None
         self.refresh_event_time = 0
         regions = self.refresh_regions
         self.refresh_regions = []
-        if self.can_refresh(window) and regions:
+        if self.can_refresh(window) and regions and ret>0:
             now = time.time()
             refreshlog("timer_full_refresh() after %ims, regions=%s", 1000.0*(time.time()-ret), regions)
             #choose an encoding:
@@ -1324,7 +1371,7 @@ class WindowSource(object):
     def webp_encode(self, coding, image, options):
         q = options.get("quality") or self.get_quality(coding)
         s = options.get("speed") or self.get_speed(coding)
-        return webp_encode(coding, image, self.supports_transparency, q, s, options)
+        return webp_encode(coding, image, self.rgb_formats, self.supports_transparency, q, s, options)
 
     def rgb_encode(self, coding, image, options):
         s = options.get("speed") or self._current_speed
