@@ -15,7 +15,7 @@ if sys.version_info < (2, 6):
 else:
     ImmutableSet = frozenset
 
-from xpra.x11.gtk_x11.error import trap
+from xpra.gtk_common.error import trap
 import xpra.x11.gtk_x11.selection
 from xpra.x11.gtk_x11.world_window import WorldWindow
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
@@ -229,6 +229,7 @@ class Wm(gobject.GObject):
         root_w, root_h = gtk.gdk.get_default_root_window().get_size()
         self.root_set("_NET_SUPPORTED", ["atom"], self._NET_SUPPORTED)
         self.set_workarea(0, 0, root_w, root_h)
+        self.set_desktop_geometry(root_w, root_h)
         self.root_set("_NET_DESKTOP_VIEWPORT", ["u32"], [0, 0])
 
         # Load up our full-screen widget
@@ -264,9 +265,19 @@ class Wm(gobject.GObject):
     def root_set(self, *args):
         prop_set(self._root, *args)
 
+    def set_dpi(self, xdpi, ydpi):
+        #this is used by some newer versions of the dummy driver (xf86-driver-dummy)
+        #(and will not be honoured by anything else..)
+        self.root_set("dummy-constant-xdpi", "u32", xdpi)
+        self.root_set("dummy-constant-ydpi", "u32", ydpi)
+
     def set_workarea(self, x, y, width, height):
         self.root_set("_NET_WORKAREA", ["u32"], [x, y, width, height])
 
+    def set_desktop_geometry(self, width, height):
+        self.root_set("_NET_DESKTOP_GEOMETRY", ["u32"], [width, height])
+
+
     def enableCursors(self, on):
         log("enableCursors(%s)" % on)
         X11Keyboard.selectCursorChange(on)
@@ -354,6 +365,9 @@ class Wm(gobject.GObject):
         self.emit("quit")
 
     def do_quit(self):
+        self.cleanup()
+
+    def cleanup(self):
         for win in list(self._windows.itervalues()):
             win.unmanage(True)
 
