@@ -6,7 +6,7 @@
 
 import gtk
 import gobject
-from xpra.x11.gtk_x11.error import trap
+from xpra.gtk_common.error import trap
 from xpra.x11.bindings.window_bindings import constants     #@UnresolvedImport
 from xpra.x11.gtk_x11.send_wm import send_wm_take_focus     #@UnresolvedImport
 from xpra.x11.gtk_x11.prop import prop_set
@@ -110,13 +110,18 @@ class WorldWindow(gtk.Window):
         self.move(0, 0)
         self._resize()
 
+    def __repr__(self):
+        xid = 0
+        if self.window:
+            xid = self.window.xid
+        return "WorldWindow(%#x)" % xid
+
     def _resize(self, *args):
         x = gtk.gdk.screen_width()
         y = gtk.gdk.screen_height()
         log("sizing world to %sx%s", x, y)
         self.set_size_request(x, y)
         self.resize(x, y)
-        root_set("_NET_DESKTOP_GEOMETRY", ["u32"], [x, y])
 
     # We want to fake GTK out into thinking that this window always has
     # toplevel focus, no matter what happens.  There are two parts to this:
@@ -148,15 +153,15 @@ class WorldWindow(gtk.Window):
             # *will* get the focus, and thus a real FocusIn event.
             send_wm_take_focus(self.window, CurrentTime)
 
-    def do_focus_in_event(self, *args):
-        focuslog("world window got focus")
+    def do_focus_in_event(self, event):
+        focuslog("world window got focus: %s", event)
         if not self.get_property("has-toplevel-focus"):
             #super(WorldWindow, self).do_focus_in_event(*args)
-            gtk.Window.do_focus_in_event(self, *args)
+            gtk.Window.do_focus_in_event(self, event)
             self.reset_x_focus()
 
-    def do_focus_out_event(self, *args):
-        focuslog("world window lost focus")
+    def do_focus_out_event(self, event):
+        focuslog("world window lost focus: %s", event)
         # Do nothing -- harder:
         self.stop_emission("focus-out-event")
         return False
