@@ -12,8 +12,10 @@ from xpra.log import Logger
 log = Logger("keyboard")
 
 
+from xpra.gtk_common.keymap import get_gtk_keymap
 from xpra.x11.gtk_x11.keys import grok_modifier_map
 from xpra.keyboard.mask import DEFAULT_MODIFIER_NUISANCE, mask_to_names
+from xpra.server.keyboard_config_base import KeyboardConfigBase
 from xpra.x11.xkbhelper import do_set_keymap, set_all_keycodes, \
                            get_modifiers_from_meanings, get_modifiers_from_keycodes, \
                            clear_modifiers, set_modifiers, \
@@ -33,8 +35,9 @@ ALL_X11_MODIFIERS = {
                     }
 
 
-class KeyboardConfig(object):
+class KeyboardConfig(KeyboardConfigBase):
     def __init__(self):
+        KeyboardConfigBase.__init__(self)
         self.xkbmap_print = None
         self.xkbmap_query = None
         self.xkbmap_mod_meanings = {}
@@ -45,7 +48,6 @@ class KeyboardConfig(object):
         self.xkbmap_layout = None
         self.xkbmap_variant = None
 
-        self.enabled = True
         #this is shared between clients!
         self.keys_pressed = {}
         #these are derived by calling set_keymap:
@@ -55,31 +57,28 @@ class KeyboardConfig(object):
         self.modifier_client_keycodes = {}
         self.compute_modifier_map()
         self.modifiers_filter = []
-        self.is_native_keymap = True
 
     def __repr__(self):
         return "KeyboardConfig(%s / %s)" % (self.xkbmap_layout, self.xkbmap_variant)
 
     def get_info(self):
-        info = {"enabled"   : self.enabled,
-                "native"    : self.is_native_keymap,
-                "modifiers.filter"          : self.modifiers_filter,
-                }
+        info = KeyboardConfigBase.get_info(self)
+        info["modifiers.filter"] = self.modifiers_filter
         #keycodes:
         if self.keycode_translation:
-            for ks, keycode in self.keycode_translation.items():
-                if type(ks)==tuple:
-                    client_keycode, keysym = ks
-                    info["keysym." + str(keysym)+"."+str(client_keycode)] = keycode
-                else:
-                    info["keysym." + str(ks)] = keycode
-        if self.xkbmap_keycodes:
-            for kc, spec in self.keycode_translation.items():
+            for kc, keycode in self.keycode_translation.items():
                 if type(kc)==tuple:
                     client_keycode, keysym = kc
-                    info["keycode." + str(client_keycode)+"."+keysym] = spec
+                    info["keysym." + str(keysym)+"."+str(client_keycode)] = keycode
+                    info["keycode." + str(client_keycode)+"."+keysym] = keycode
                 else:
-                    info["keycode." + str(kc)] = spec
+                    info["keysym." + str(kc)] = keycode
+                    info["keycode." + str(kc)] = keycode
+        if self.xkbmap_keycodes:
+            i = 0
+            for keyval, name, keycode, group, level in self.xkbmap_keycodes:
+                info["keymap.%s" % i] = (keyval, name, keycode, group, level)
+                i += 1
         #modifiers:
         if self.modifier_client_keycodes:
             for mod, keys in self.modifier_client_keycodes.items():
@@ -108,7 +107,28 @@ class KeyboardConfig(object):
         return info
 
 
+    def parse_options(self, props):
+        """ used by both process_hello and process_keymap
+            to set the keyboard attributes """
+        KeyboardConfigBase.parse_options(self, props)
+        modded = []
+        for x in ("print", "query", "mod_meanings",
+                  "mod_managed", "mod_pointermissing",
+                  "keycodes", "x11_keycodes"):
+            prop = "xkbmap_%s" % x
+            cv = getattr(self, prop)
+            nv = props.get(prop)
+            if cv!=nv:
+                setattr(self, prop, nv)
+                modded.append(prop)
+        log("assign_keymap_options(..) modified %s", modded)
+        return len(modded)>0
+
+
     def get_hash(self):
+        """
+            This hash will be different whenever the keyboard configuration changes.
+        """
         try:
             import hashlib
             m = hashlib.sha1()
@@ -116,6 +136,7 @@ class KeyboardConfig(object):
             #try python2.4 variant:
             import sha
             m = sha.new()
+        m.update(KeyboardConfigBase.get_hash(self))
         for x in (self.xkbmap_print, self.xkbmap_query, \
                   self.xkbmap_mod_meanings, self.xkbmap_mod_pointermissing, \
                   self.xkbmap_keycodes, self.xkbmap_x11_keycodes):
@@ -164,7 +185,16 @@ class KeyboardConfig(object):
         log("modifier_map(%s)=%s", self.xkbmap_mod_meanings, self.modifier_map)
 
 
-    def set_keymap(self, client_platform):
+    def is_modifier(self, keycode):
+        for mod, keys in self.keycodes_for_modifier_keynames.items():
+            if keycode in keys:
+                log("is_modifier(%s) found modifier: %s", keycode, mod)
+                return True
+        log("is_modifier(%s) not found", keycode)
+        return False
+
+
+    def set_keymap(self):
         if not self.enabled:
             return
         clean_keyboard_state()
@@ -202,6 +232,7 @@ class KeyboardConfig(object):
             #(used by non X11 clients like osx,win32 or Android)
             preserve_server_keycodes = not self.xkbmap_print and not self.xkbmap_query
             self.keycode_translation = set_all_keycodes(self.xkbmap_x11_keycodes, self.xkbmap_keycodes, preserve_server_keycodes, self.keynames_for_mod)
+            self.add_gtk_keynames()
 
             #now set the new modifier mappings:
             clean_keyboard_state()
@@ -214,6 +245,71 @@ class KeyboardConfig(object):
         except:
             log.error("error setting xmodmap", exc_info=True)
 
+    def add_gtk_keynames(self):
+        #add the keynames we find via gtk
+        #since we may rely on finding those keynames from the client
+        #(used with non native keymaps)
+        for _, keyname, keycode, _, _ in get_gtk_keymap():
+            if keyname not in self.keycode_translation:
+                self.keycode_translation[keyname] = keycode
+
+    def set_default_keymap(self):
+        """ assign a default keymap based on the current X11 server keymap
+            sets up the translation tables so we can lookup keys without
+            setting a client keymap.
+        """
+        if not self.enabled:
+            return
+        self.is_native_keymap = False
+        clean_keyboard_state()
+        #keycodes:
+        keycode_to_keynames = X11Keyboard.get_keycode_mappings()
+        self.keycode_translation = {}
+        for keycode, keynames in keycode_to_keynames.items():
+            for keyname in keynames:
+                self.keycode_translation[keyname] = keycode
+        self.add_gtk_keynames()
+        log("set_default_keymap: keycode_translation=%s", self.keycode_translation)
+        #modifiers:
+        self.keynames_for_mod = {}
+        #ie: {'control': [(37, 'Control_L'), (105, 'Control_R')], ...}
+        mod_mappings = X11Keyboard.get_modifier_mappings()
+        log("set_default_keymap: using modifier mappings=%s", mod_mappings)
+        for modifier, mappings in mod_mappings.items():
+            keynames = []
+            for m in mappings:      #ie: (37, 'Control_L'), (105, 'Control_R')
+                if len(m)==2:
+                    keynames.append(m[1])   #ie: 'Control_L'
+            self.keynames_for_mod[modifier] = set(keynames)
+        self.compute_modifier_keynames()
+        log("set_default_keymap: keynames_for_mod=%s", self.keynames_for_mod)
+        log("set_default_keymap: keycodes_for_modifier_keynames=%s", self.keycodes_for_modifier_keynames)
+        log("set_default_keymap: modifier_map=%s", self.modifier_map)
+
+
+    def get_keycode(self, client_keycode, keyname, modifiers):
+        if not self.enabled:
+            log("ignoring keycode since keyboard is turned off")
+            return -1
+        keycode = self.keycode_translation.get((client_keycode, keyname))
+        if keycode is None:
+            if self.is_native_keymap:
+                #native: assume no translation for this key
+                keycode = client_keycode
+                log("get_keycode(%s, %s, %s) native keymap, using unmodified keycode: %s", client_keycode, keyname, modifiers, keycode)
+            else:
+                #non-native: try harder to find matching keysym
+                keycode = self.keycode_translation.get(keyname, client_keycode)
+                log("get_keycode(%s, %s, %s) non-native keymap, translation lookup: %s", client_keycode, keyname, modifiers, keycode)
+        else:
+            log("get_keycode(%s, %s, %s) is_native_keymap=%s, found using translation: %s", client_keycode, keyname, modifiers, self.is_native_keymap, keycode)
+        return keycode
+
+
+    def get_current_mask(self):
+        _, _, current_mask = gtk.gdk.get_default_root_window().get_pointer()
+        return mask_to_names(current_mask, self.modifier_map)
+
     def make_keymask_match(self, modifier_list, ignored_modifier_keycode=None, ignored_modifier_keynames=None):
         """
             Given a list of modifiers that should be set, try to press the right keys
@@ -225,36 +321,35 @@ class KeyboardConfig(object):
                 ie: "num" on win32, which is toggled by the "Num_Lock" key presses.
             * when called from '_handle_key', we ignore the modifier key which may be pressed
                 or released as it should be set by that key press event.
-            * when called from mouse position/click events we ignore 'xkbmap_mod_pointermissing'
+            * when called from mouse position/click/focus events we ignore 'xkbmap_mod_pointermissing'
                 which is set by the client to indicate modifiers which are missing from mouse events.
                 ie: on win32, "lock" is missing.
+                (we know this is not a keyboard event because ignored_modifier_keynames is None..)
             * if the modifier is a "nuisance" one ("lock", "num", "scroll") then we must
                 simulate a full keypress (down then up).
             * some modifiers can be set by multiple keys ("shift" by both "Shift_L" and "Shift_R" for example)
                 so we try to find the matching modifier in the currently pressed keys (keys_pressed)
                 to make sure we unpress the right one.
         """
-        def get_current_mask():
-            _, _, current_mask = gtk.gdk.get_default_root_window().get_pointer()
-            return mask_to_names(current_mask, self.modifier_map)
-
         if not self.keynames_for_mod:
             log("make_keymask_match: ignored as keynames_for_mod not assigned yet")
             return
         if ignored_modifier_keynames is None:
-            ignored_modifier_keynames = self.xkbmap_mod_pointermissing
+            #this is not a keyboard event, ignore modifiers in "mod_pointermissing"
+            def is_ignored(modifier, modifier_keynames):
+                m = modifier in self.xkbmap_mod_pointermissing
+                log("is_ignored(%s, %s)=%s", modifier, modifier_keynames, m)
+                return m
+        else:
+            #keyboard event: ignore the keynames specified
+            #(usually the modifier key being pressed/unpressed)
+            def is_ignored(modifier, modifier_keynames):
+                m = set(modifier_keynames) & set(ignored_modifier_keynames)
+                log("is_ignored(%s, %s)=%s", modifier, modifier_keynames, m)
+                return bool(m)
 
-        def is_ignored(modifier_keynames):
-            if not ignored_modifier_keynames:
-                return False
-            for imk in ignored_modifier_keynames:
-                if imk in modifier_keynames:
-                    log("modifier ignored (ignored keyname=%s)", imk)
-                    return True
-            return False
-
-        current = set(get_current_mask())
-        wanted = set(modifier_list)
+        current = set(self.get_current_mask())
+        wanted = set(modifier_list or [])
         if current==wanted:
             return
         log("make_keymask_match(%s) current mask: %s, wanted: %s, ignoring=%s/%s, keys_pressed=%s", modifier_list, current, wanted, ignored_modifier_keycode, ignored_modifier_keynames, self.keys_pressed)
@@ -268,7 +363,7 @@ class KeyboardConfig(object):
                 if not keynames:
                     log.error("unknown modifier: %s", modifier)
                     continue
-                if is_ignored(keynames):
+                if is_ignored(modifier, keynames):
                     log("modifier %s ignored (in ignored keynames=%s)", modifier, keynames)
                     continue
                 #find the keycodes that match the keynames for this modifier
@@ -299,7 +394,7 @@ class KeyboardConfig(object):
                         X11Keyboard.xtest_fake_key(keycode, False)
                     else:
                         X11Keyboard.xtest_fake_key(keycode, press)
-                    new_mask = get_current_mask()
+                    new_mask = self.get_current_mask()
                     success = (modifier in new_mask)==press
                     log("make_keymask_match(%s) %s modifier %s using %s, success: %s", info, modifier_list, modifier, keycode, success)
                     if success:
@@ -307,7 +402,7 @@ class KeyboardConfig(object):
                     elif not nuisance:
                         log("%s %s with keycode %s did not work - trying to undo it!", info, modifier, keycode)
                         X11Keyboard.xtest_fake_key(keycode, not press)
-                        new_mask = get_current_mask()
+                        new_mask = self.get_current_mask()
                         #maybe doing the full keypress (down+up or u+down) worked:
                         if (modifier in new_mask)==press:
                             break
