@@ -28,6 +28,7 @@ log = Logger("x11", "server")
 keylog = Logger("x11", "server", "keyboard")
 mouselog = Logger("x11", "server", "mouse")
 grablog = Logger("server", "grab")
+cursorlog = Logger("server", "cursor")
 
 from xpra.util import prettify_plug_name
 from xpra.server.gtk_server_base import GTKServerBase
@@ -212,11 +213,8 @@ class X11ServerBase(GTKServerBase):
 
     def _clear_keys_pressed(self):
         keylog("_clear_keys_pressed()")
-        #make sure the timers don't fire and interfere:
-        if len(self.keys_repeat_timers)>0:
-            for timer in self.keys_repeat_timers.values():
-                gobject.source_remove(timer)
-            self.keys_repeat_timers = {}
+        #make sure the timer doesn't fire and interfere:
+        self.cancel_key_repeat_timer()
         #clear all the keys we know about:
         if len(self.keys_pressed)>0:
             log("clearing keys pressed: %s", self.keys_pressed)
@@ -228,6 +226,23 @@ class X11ServerBase(GTKServerBase):
         X11Keyboard.unpress_all_keys()
 
 
+    def get_cursor_data(self):
+        #must be called from the UI thread!
+        cursor_data = trap.call(X11Keyboard.get_cursor_image)
+        if cursor_data is None:
+            cursorlog("get_cursor_data() failed to get cursor image")
+            return None, []
+        self.last_cursor_data = cursor_data
+        pixels = self.last_cursor_data[7]
+        cursorlog("get_cursor_data() cursor=%s", cursor_data[:7]+["%s bytes" % len(pixels)]+cursor_data[8:])
+        if self.default_cursor_data is not None and str(pixels)==str(self.default_cursor_data[7]):
+            cursorlog("get_cursor_data(): default cursor - clearing it")
+            cursor_data = None
+        display = gtk.gdk.display_get_default()
+        cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
+        return (cursor_data, cursor_sizes)
+
+
     def get_max_screen_size(self):
         from xpra.x11.gtk_x11.window import MAX_WINDOW_SIZE
         max_w, max_h = gtk.gdk.get_default_root_window().get_size()
@@ -384,7 +399,7 @@ class X11ServerBase(GTKServerBase):
             return delfile(None)
         ui_client_count = len([True for s in self._server_sources.values() if s.ui_client])
         if ui_client_count!=1:
-            return delfile("fakeXinerama can only be enabled for a single client (found %s)", ui_client_count)
+            return delfile("fakeXinerama can only be enabled for a single client (found %s)" % ui_client_count)
         source = self._server_sources.values()[0]
         ss = source.screen_sizes
         if len(ss)==0:
