@@ -7,7 +7,7 @@
 import gobject
 import gtk
 from xpra.gtk_common.gobject_util import no_arg_signal, one_arg_signal
-from xpra.x11.gtk_x11.error import trap, XError
+from xpra.gtk_common.error import trap, XError
 from xpra.x11.gtk_x11.selection import ManagerSelection
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
 
@@ -44,7 +44,9 @@ class XSettingsManager(object):
         self._window = self._manager.window()
 
     def set_settings(self, settings):
-        if type(settings)!=tuple:
+        if type(settings)==list:
+            settings = tuple(settings)
+        elif type(settings)!=tuple:
             log.warn("discarding xsettings because of incompatible format: %s", type(settings))
             return
         prop_set(self._window, XSETTINGS, XSETTINGS_TYPE, settings)
